@@ -213,6 +213,39 @@ async fn large_filename() {
     assert!(entries.next().await.is_none());
 }
 
+// This test checks very particular scenario where path component
+// starting with ".." of a long path gets split at 100-byte mark
+// so that ".." goes into header and gets interpreted as parent dir
+// (and rejected) .
+#[async_std::test]
+async fn large_filename_with_dot_dot_at_100_byte_mark() {
+    let mut ar = Builder::new(Vec::new());
+
+    let mut header = Header::new_gnu();
+    header.set_cksum();
+    header.set_mode(0o644);
+    header.set_size(4);
+
+    let mut long_name_with_dot_dot = "tdir/".repeat(19);
+    long_name_with_dot_dot.push_str("tt/..file");
+
+    t!(ar
+        .append_data(&mut header, &long_name_with_dot_dot, &b"test"[..])
+        .await);
+
+    let rd = Cursor::new(t!(ar.into_inner().await));
+    let ar = Archive::new(rd);
+    let mut entries = t!(ar.entries());
+
+    let mut f = entries.next().await.unwrap().unwrap();
+    assert_eq!(&*f.path_bytes(), long_name_with_dot_dot.as_bytes());
+    assert_eq!(f.header().size().unwrap(), 4);
+    let mut s = String::new();
+    t!(f.read_to_string(&mut s).await);
+    assert_eq!(s, "test");
+    assert!(entries.next().await.is_none());
+}
+
 #[async_std::test]
 async fn reading_entries() {
     let rdr = Cursor::new(tar!("reading_files.tar"));
@@ -239,18 +272,24 @@ async fn check_dirtree(td: &TempDir) {
     let dir_a = td.path().join("a");
     let dir_b = td.path().join("a/b");
     let file_c = td.path().join("a/c");
-    assert!(fs::metadata(&dir_a)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
-    assert!(fs::metadata(&dir_b)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
-    assert!(fs::metadata(&file_c)
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&dir_a)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
+    assert!(
+        fs::metadata(&dir_b)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
+    assert!(
+        fs::metadata(&file_c)
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 }
 
 #[async_std::test]
@@ -340,25 +379,33 @@ async fn writing_directories_recursively() {
     let ar = Archive::new(Cursor::new(data));
     t!(ar.unpack(td.path()).await);
     let base_dir = td.path().join("foobar");
-    assert!(fs::metadata(&base_dir)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&base_dir)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
     let file1_path = base_dir.join("file1");
-    assert!(fs::metadata(&file1_path)
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&file1_path)
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
     let sub_dir = base_dir.join("sub");
-    assert!(fs::metadata(&sub_dir)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&sub_dir)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
     let file2_path = sub_dir.join("file2");
-    assert!(fs::metadata(&file2_path)
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&file2_path)
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 }
 
 #[async_std::test]
@@ -383,25 +430,33 @@ async fn append_dir_all_blank_dest() {
     let ar = Archive::new(Cursor::new(data));
     t!(ar.unpack(td.path()).await);
     let base_dir = td.path();
-    assert!(fs::metadata(&base_dir)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&base_dir)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
     let file1_path = base_dir.join("file1");
-    assert!(fs::metadata(&file1_path)
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&file1_path)
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
     let sub_dir = base_dir.join("sub");
-    assert!(fs::metadata(&sub_dir)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&sub_dir)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
     let file2_path = sub_dir.join("file2");
-    assert!(fs::metadata(&file2_path)
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&file2_path)
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 }
 
 #[async_std::test]
@@ -423,10 +478,12 @@ async fn extracting_duplicate_dirs() {
     t!(ar.unpack(td.path()).await);
 
     let some_dir = td.path().join("some_dir");
-    assert!(fs::metadata(&some_dir)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&some_dir)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
 }
 
 #[async_std::test]
@@ -532,18 +589,26 @@ async fn extracting_malicious_tarball() {
     assert!(fs::metadata("/tmp/abs_evil.txt6").await.is_err());
     assert!(fs::metadata("/tmp/rel_evil.txt").await.is_err());
     assert!(fs::metadata("/tmp/rel_evil.txt").await.is_err());
-    assert!(fs::metadata(td.path().join("../tmp/rel_evil.txt"))
-        .await
-        .is_err());
-    assert!(fs::metadata(td.path().join("../rel_evil2.txt"))
-        .await
-        .is_err());
-    assert!(fs::metadata(td.path().join("../rel_evil3.txt"))
-        .await
-        .is_err());
-    assert!(fs::metadata(td.path().join("../rel_evil4.txt"))
-        .await
-        .is_err());
+    assert!(
+        fs::metadata(td.path().join("../tmp/rel_evil.txt"))
+            .await
+            .is_err()
+    );
+    assert!(
+        fs::metadata(td.path().join("../rel_evil2.txt"))
+            .await
+            .is_err()
+    );
+    assert!(
+        fs::metadata(td.path().join("../rel_evil3.txt"))
+            .await
+            .is_err()
+    );
+    assert!(
+        fs::metadata(td.path().join("../rel_evil4.txt"))
+            .await
+            .is_err()
+    );
 
     // The `some` subdirectory should not be created because the only
     // filename that references this has '..'.
@@ -554,10 +619,12 @@ async fn extracting_malicious_tarball() {
     // `abs_evil6.txt`.
     let tmp_root = td.path().join("tmp");
 
-    assert!(fs::metadata(&tmp_root)
-        .await
-        .map(|m| m.is_dir())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(&tmp_root)
+            .await
+            .map(|m| m.is_dir())
+            .unwrap_or(false)
+    );
 
     let mut entries = fs::read_dir(&tmp_root).await.unwrap();
     while let Some(entry) = entries.next().await {
@@ -565,36 +632,48 @@ async fn extracting_malicious_tarball() {
         println!("- {:?}", entry.file_name());
     }
 
-    assert!(fs::metadata(tmp_root.join("abs_evil.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 
     // not present due to // being interpreted differently on windows
     #[cfg(not(target_os = "windows"))]
-    assert!(fs::metadata(tmp_root.join("abs_evil2.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
-    assert!(fs::metadata(tmp_root.join("abs_evil3.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
-    assert!(fs::metadata(tmp_root.join("abs_evil4.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil2.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil3.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil4.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 
     // not present due to // being interpreted differently on windows
     #[cfg(not(target_os = "windows"))]
-    assert!(fs::metadata(tmp_root.join("abs_evil5.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
-    assert!(fs::metadata(tmp_root.join("abs_evil6.txt"))
-        .await
-        .map(|m| m.is_file())
-        .unwrap_or(false));
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil5.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
+    assert!(
+        fs::metadata(tmp_root.join("abs_evil6.txt"))
+            .await
+            .map(|m| m.is_file())
+            .unwrap_or(false)
+    );
 }
 
 #[async_std::test]
@@ -766,6 +845,23 @@ async fn pax_path() {
     assert!(first.path().unwrap().ends_with("aaaaaaaaaaaaaaa"));
 }
 
+#[async_std::test]
+async fn pax_precedence() {
+    let mut ar = Archive::new(tar!("pax-header-precedence.tar"));
+    let mut entries = t!(ar.entries());
+
+    let first = t!(entries.next().await.unwrap());
+    assert!(first.path().unwrap().ends_with("normal.txt"));
+
+    let second = t!(entries.next().await.unwrap());
+    assert!(second.path().unwrap().ends_with("blob.bin"));
+
+    let third = t!(entries.next().await.unwrap());
+    assert!(third.path().unwrap().ends_with("marker.txt"));
+
+    assert!(entries.next().await.is_none());
+}
+
 #[async_std::test]
 async fn long_name_trailing_nul() {
     let mut b = Builder::new(Vec::<u8>::new());
