@@ -1,18 +1,10 @@
-use objc2_audio_toolbox::{
-    kAudioOutputUnitProperty_SetInputCallback, kAudioUnitProperty_SetRenderCallback,
-    kAudioUnitProperty_StreamFormat, AURenderCallbackStruct, AudioUnitRender,
-    AudioUnitRenderActionFlags,
-};
-use objc2_core_audio_types::{AudioBuffer, AudioBufferList, AudioTimeStamp};
-
 use super::audio_format::LinearPcmFlags;
 use super::{AudioUnit, Element, Scope};
 use crate::error::{self, Error};
-use crate::OSStatus;
 use std::mem;
 use std::os::raw::c_void;
-use std::ptr::NonNull;
 use std::slice;
+use sys;
 
 pub use self::action_flags::ActionFlags;
 pub use self::data::Data;
@@ -23,12 +15,12 @@ pub use self::data::Data;
 /// This allows the user to provide a custom, more rust-esque callback function type that takes
 /// greater advantage of rust's type safety.
 pub type InputProcFn = dyn FnMut(
-    NonNull<AudioUnitRenderActionFlags>,
-    NonNull<AudioTimeStamp>,
-    u32,
-    u32,
-    *mut AudioBufferList,
-) -> OSStatus;
+    *mut sys::AudioUnitRenderActionFlags,
+    *const sys::AudioTimeStamp,
+    sys::UInt32,
+    sys::UInt32,
+    *mut sys::AudioBufferList,
+) -> sys::OSStatus;
 
 /// This type allows us to safely wrap a boxed `RenderCallback` to use within the input proc.
 pub struct InputProcFnWrapper {
@@ -41,7 +33,7 @@ pub struct Args<D> {
     /// A type wrapping the the buffer that matches the expected audio format.
     pub data: D,
     /// Timing information for the callback.
-    pub time_stamp: AudioTimeStamp,
+    pub time_stamp: sys::AudioTimeStamp,
     /// TODO
     pub bus_number: u32,
     /// The number of frames in the buffer as `usize` for easier indexing.
@@ -57,36 +49,38 @@ pub struct Args<D> {
 
 /// Format specific render callback data.
 pub mod data {
-    use objc2_core_audio_types::AudioBuffer;
-    use objc2_core_audio_types::AudioBufferList;
-
     use super::super::Sample;
     use super::super::StreamFormat;
     use crate::audio_unit::audio_format::LinearPcmFlags;
     use std::marker::PhantomData;
     use std::slice;
+    use sys;
 
     /// Audio data wrappers specific to the `AudioUnit`'s `AudioFormat`.
     pub trait Data {
-        /// Check whether the stream format matches this type of data.
+        /// Check whether or not the stream format matches this type of data.
         fn does_stream_format_match(stream_format: &StreamFormat) -> bool;
         /// We must be able to construct Self from arguments given to the `input_proc`.
         /// # Safety
         /// TODO document how to use this function safely.
-        unsafe fn from_input_proc_args(num_frames: u32, io_data: *mut AudioBufferList) -> Self;
+        unsafe fn from_input_proc_args(num_frames: u32, io_data: *mut sys::AudioBufferList)
+            -> Self;
     }
 
     /// A raw pointer to the audio data so that the user may handle it themselves.
     #[derive(Debug)]
     pub struct Raw {
-        pub data: *mut AudioBufferList,
+        pub data: *mut sys::AudioBufferList,
     }
 
     impl Data for Raw {
         fn does_stream_format_match(_: &StreamFormat) -> bool {
             true
         }
-        unsafe fn from_input_proc_args(_num_frames: u32, io_data: *mut AudioBufferList) -> Self {
+        unsafe fn from_input_proc_args(
+            _num_frames: u32,
+            io_data: *mut sys::AudioBufferList,
+        ) -> Self {
             Raw { data: io_data }
         }
     }
@@ -110,7 +104,7 @@ pub mod data {
     /// A wrapper around the pointer to the `mBuffers` array.
     pub struct NonInterleaved<S> {
         /// The list of audio buffers.
-        buffers: &'static mut [AudioBuffer],
+        buffers: &'static mut [sys::AudioBuffer],
         /// The number of frames in each channel.
         frames: usize,
         sample_format: PhantomData<S>,
@@ -118,14 +112,14 @@ pub mod data {
 
     /// An iterator produced by a `NonInterleaved`, yielding a reference to each channel.
     pub struct Channels<'a, S: 'a> {
-        buffers: slice::Iter<'a, AudioBuffer>,
+        buffers: slice::Iter<'a, sys::AudioBuffer>,
         frames: usize,
         sample_format: PhantomData<S>,
     }
 
     /// An iterator produced by a `NonInterleaved`, yielding a mutable reference to each channel.
     pub struct ChannelsMut<'a, S: 'a> {
-        buffers: slice::IterMut<'a, AudioBuffer>,
+        buffers: slice::IterMut<'a, sys::AudioBuffer>,
         frames: usize,
         sample_format: PhantomData<S>,
     }
@@ -137,7 +131,7 @@ pub mod data {
         #[allow(non_snake_case)]
         fn next(&mut self) -> Option<Self::Item> {
             self.buffers.next().map(
-                |&AudioBuffer {
+                |&sys::AudioBuffer {
                      mNumberChannels,
                      mData,
                      ..
@@ -155,7 +149,7 @@ pub mod data {
         #[allow(non_snake_case)]
         fn next(&mut self) -> Option<Self::Item> {
             self.buffers.next().map(
-                |&mut AudioBuffer {
+                |&mut sys::AudioBuffer {
                      mNumberChannels,
                      mData,
                      ..
@@ -201,8 +195,8 @@ pub mod data {
         }
 
         #[allow(non_snake_case)]
-        unsafe fn from_input_proc_args(frames: u32, io_data: *mut AudioBufferList) -> Self {
-            let ptr = (*io_data).mBuffers.as_ptr() as *mut AudioBuffer;
+        unsafe fn from_input_proc_args(frames: u32, io_data: *mut sys::AudioBufferList) -> Self {
+            let ptr = (*io_data).mBuffers.as_ptr() as *mut sys::AudioBuffer;
             let len = (*io_data).mNumberBuffers as usize;
             let buffers = slice::from_raw_parts_mut(ptr, len);
             NonInterleaved {
@@ -226,9 +220,9 @@ pub mod data {
         }
 
         #[allow(non_snake_case)]
-        unsafe fn from_input_proc_args(frames: u32, io_data: *mut AudioBufferList) -> Self {
+        unsafe fn from_input_proc_args(frames: u32, io_data: *mut sys::AudioBufferList) -> Self {
             // // We're expecting a single interleaved buffer which will be the first in the array.
-            let AudioBuffer {
+            let sys::AudioBuffer {
                 mNumberChannels,
                 mDataByteSize,
                 mData,
@@ -267,9 +261,9 @@ pub mod data {
         }
 
         #[allow(non_snake_case)]
-        unsafe fn from_input_proc_args(frames: u32, io_data: *mut AudioBufferList) -> Self {
+        unsafe fn from_input_proc_args(frames: u32, io_data: *mut sys::AudioBufferList) -> Self {
             // // We're expecting a single interleaved buffer which will be the first in the array.
-            let AudioBuffer {
+            let sys::AudioBuffer {
                 mNumberChannels,
                 mDataByteSize,
                 mData,
@@ -297,9 +291,8 @@ pub mod data {
 }
 
 pub mod action_flags {
-    use objc2_audio_toolbox::AudioUnitRenderActionFlags;
-
     use std::fmt;
+    use sys;
 
     bitflags! {
         pub struct ActionFlags: u32 {
@@ -308,20 +301,20 @@ pub mod action_flags {
             /// before the render operation is performed.
             ///
             /// **Available** in OS X v10.0 and later.
-            const PRE_RENDER = AudioUnitRenderActionFlags::UnitRenderAction_PreRender.0;
+            const PRE_RENDER = sys::kAudioUnitRenderAction_PreRender;
             /// Called on a render notification Proc, which is called either before or after the
             /// render operation of the audio unit. If this flag is set, the proc is being called
             /// after the render operation is completed.
             ///
             /// **Available** in OS X v10.0 and later.
-            const POST_RENDER = AudioUnitRenderActionFlags::UnitRenderAction_PostRender.0;
+            const POST_RENDER = sys::kAudioUnitRenderAction_PostRender;
             /// This flag can be set in a render input callback (or in the audio unit's render
             /// operation itself) and is used to indicate that the render buffer contains only
             /// silence. It can then be used by the caller as a hint to whether the buffer needs to
             /// be processed or not.
             ///
             /// **Available** in OS X v10.2 and later.
-            const OUTPUT_IS_SILENCE = AudioUnitRenderActionFlags::UnitRenderAction_OutputIsSilence.0;
+            const OUTPUT_IS_SILENCE = sys::kAudioUnitRenderAction_OutputIsSilence;
             /// This is used with offline audio units (of type 'auol'). It is used when an offline
             /// unit is being preflighted, which is performed prior to when the actual offline
             /// rendering actions are performed. It is used for those cases where the offline
@@ -330,32 +323,32 @@ pub mod action_flags {
             /// normalization).
             ///
             /// **Available** in OS X v10.3 and later.
-            const OFFLINE_PREFLIGHT = AudioUnitRenderActionFlags::OfflineUnitRenderAction_Preflight.0;
+            const OFFLINE_PREFLIGHT = sys::kAudioOfflineUnitRenderAction_Preflight;
             /// Once an offline unit has been successfully preflighted, it is then put into its
             /// render mode. This flag is set to indicate to the audio unit that it is now in that
             /// state and that it should perform processing on the input data.
             ///
             /// **Available** in OS X v10.3 and later.
-            const OFFLINE_RENDER = AudioUnitRenderActionFlags::OfflineUnitRenderAction_Render.0;
+            const OFFLINE_RENDER = sys::kAudioOfflineUnitRenderAction_Render;
             /// This flag is set when an offline unit has completed either its preflight or
             /// performed render operation.
             ///
             /// **Available** in OS X v10.3 and later.
-            const OFFLINE_COMPLETE = AudioUnitRenderActionFlags::OfflineUnitRenderAction_Complete.0;
+            const OFFLINE_COMPLETE = sys::kAudioOfflineUnitRenderAction_Complete;
             /// If this flag is set on the post-render call an error was returned by the audio
             /// unit's render operation. In this case, the error can be retrieved through the
             /// `lastRenderError` property and the audio data in `ioData` handed to the post-render
             /// notification will be invalid.
             ///
             /// **Available** in OS X v10.5 and later.
-            const POST_RENDER_ERROR = AudioUnitRenderActionFlags::UnitRenderAction_PostRenderError.0;
+            const POST_RENDER_ERROR = sys::kAudioUnitRenderAction_PostRenderError;
             /// If this flag is set, then checks that are done on the arguments provided to render
             /// are not performed. This can be useful to use to save computation time in situations
             /// where you are sure you are providing the correct arguments and structures to the
             /// various render calls.
             ///
             /// **Available** in OS X v10.7 and later.
-            const DO_NOT_CHECK_RENDER_ARGS = AudioUnitRenderActionFlags::UnitRenderAction_DoNotCheckRenderArgs.0;
+            const DO_NOT_CHECK_RENDER_ARGS = sys::kAudioUnitRenderAction_DoNotCheckRenderArgs;
         }
     }
 
@@ -367,7 +360,7 @@ pub mod action_flags {
     /// For example: if there is no audio to process, we can insert the `OUTPUT_IS_SILENCE` flag to
     /// indicate to the audio unit that the buffer does not need to be processed.
     pub struct Handle {
-        ptr: *mut AudioUnitRenderActionFlags,
+        ptr: *mut sys::AudioUnitRenderActionFlags,
     }
 
     impl fmt::Debug for Handle {
@@ -383,11 +376,11 @@ pub mod action_flags {
     impl Handle {
         /// Retrieve the current state of the `ActionFlags`.
         pub fn get(&self) -> ActionFlags {
-            ActionFlags::from_bits_truncate(unsafe { *self.ptr }.0)
+            ActionFlags::from_bits_truncate(unsafe { *self.ptr })
         }
 
         fn set(&mut self, flags: ActionFlags) {
-            unsafe { (*self.ptr).0 = flags.bits() }
+            unsafe { *self.ptr = flags.bits() }
         }
 
         /// The raw value of the flags currently stored.
@@ -437,7 +430,7 @@ pub mod action_flags {
         }
 
         /// Wrap the given pointer with a `Handle`.
-        pub fn from_ptr(ptr: *mut AudioUnitRenderActionFlags) -> Self {
+        pub fn from_ptr(ptr: *mut sys::AudioUnitRenderActionFlags) -> Self {
             Handle { ptr }
         }
     }
@@ -449,20 +442,15 @@ pub mod action_flags {
             write!(
                 f,
                 "{:?}",
-                match AudioUnitRenderActionFlags(self.bits()) {
-                    AudioUnitRenderActionFlags::UnitRenderAction_PreRender => "PRE_RENDER",
-                    AudioUnitRenderActionFlags::UnitRenderAction_PostRender => "POST_RENDER",
-                    AudioUnitRenderActionFlags::UnitRenderAction_OutputIsSilence =>
-                        "OUTPUT_IS_SILENCE",
-                    AudioUnitRenderActionFlags::OfflineUnitRenderAction_Preflight =>
-                        "OFFLINE_PREFLIGHT",
-                    AudioUnitRenderActionFlags::OfflineUnitRenderAction_Render => "OFFLINE_RENDER",
-                    AudioUnitRenderActionFlags::OfflineUnitRenderAction_Complete =>
-                        "OFFLINE_COMPLETE",
-                    AudioUnitRenderActionFlags::UnitRenderAction_PostRenderError =>
-                        "POST_RENDER_ERROR",
-                    AudioUnitRenderActionFlags::UnitRenderAction_DoNotCheckRenderArgs =>
-                        "DO_NOT_CHECK_RENDER_ARGS",
+                match self.bits() {
+                    sys::kAudioUnitRenderAction_PreRender => "PRE_RENDER",
+                    sys::kAudioUnitRenderAction_PostRender => "POST_RENDER",
+                    sys::kAudioUnitRenderAction_OutputIsSilence => "OUTPUT_IS_SILENCE",
+                    sys::kAudioOfflineUnitRenderAction_Preflight => "OFFLINE_PREFLIGHT",
+                    sys::kAudioOfflineUnitRenderAction_Render => "OFFLINE_RENDER",
+                    sys::kAudioOfflineUnitRenderAction_Complete => "OFFLINE_COMPLETE",
+                    sys::kAudioUnitRenderAction_PostRenderError => "POST_RENDER_ERROR",
+                    sys::kAudioUnitRenderAction_DoNotCheckRenderArgs => "DO_NOT_CHECK_RENDER_ARGS",
                     _ => "<Unknown ActionFlags>",
                 }
             )
@@ -491,18 +479,18 @@ impl AudioUnit {
         //
         // This allows us to take advantage of rust's type system and provide format-specific
         // `Args` types which can be checked at compile time.
-        let input_proc_fn = move |io_action_flags: NonNull<AudioUnitRenderActionFlags>,
-                                  in_time_stamp: NonNull<AudioTimeStamp>,
-                                  in_bus_number: u32,
-                                  in_number_frames: u32,
-                                  io_data: *mut AudioBufferList|
-              -> OSStatus {
+        let input_proc_fn = move |io_action_flags: *mut sys::AudioUnitRenderActionFlags,
+                                  in_time_stamp: *const sys::AudioTimeStamp,
+                                  in_bus_number: sys::UInt32,
+                                  in_number_frames: sys::UInt32,
+                                  io_data: *mut sys::AudioBufferList|
+              -> sys::OSStatus {
             let args = unsafe {
                 let data = D::from_input_proc_args(in_number_frames, io_data);
-                let flags = action_flags::Handle::from_ptr(io_action_flags.as_ptr());
+                let flags = action_flags::Handle::from_ptr(io_action_flags);
                 Args {
                     data,
-                    time_stamp: in_time_stamp.read(),
+                    time_stamp: *in_time_stamp,
                     flags,
                     bus_number: in_bus_number as u32,
                     num_frames: in_number_frames as usize,
@@ -525,13 +513,13 @@ impl AudioUnit {
         // within our AudioUnit's Drop implementation (otherwise it would leak).
         let input_proc_fn_wrapper_ptr = Box::into_raw(input_proc_fn_wrapper) as *mut c_void;
 
-        let render_callback = AURenderCallbackStruct {
+        let render_callback = sys::AURenderCallbackStruct {
             inputProc: Some(input_proc),
             inputProcRefCon: input_proc_fn_wrapper_ptr,
         };
 
         self.set_property(
-            kAudioUnitProperty_SetRenderCallback,
+            sys::kAudioUnitProperty_SetRenderCallback,
             Scope::Input,
             Element::Output,
             Some(&render_callback),
@@ -567,15 +555,15 @@ impl AudioUnit {
         // First, get the current buffer size for pre-allocating the `AudioBuffer`s.
         #[cfg(target_os = "macos")]
         let mut buffer_frame_size: u32 = {
-            let id = objc2_core_audio::kAudioDevicePropertyBufferFrameSize;
+            let id = sys::kAudioDevicePropertyBufferFrameSize;
             let buffer_frame_size: u32 = self.get_property(id, Scope::Global, Element::Output)?;
             buffer_frame_size
         };
         #[cfg(target_os = "ios")]
         let mut buffer_frame_size: u32 = {
-            let id = objc2_audio_toolbox::kAudioSessionProperty_CurrentHardwareIOBufferDuration;
+            let id = sys::kAudioSessionProperty_CurrentHardwareIOBufferDuration;
             let seconds: f32 = super::audio_session_get_property(id)?;
-            let id = objc2_audio_toolbox::kAudioSessionProperty_CurrentHardwareSampleRate;
+            let id = sys::kAudioSessionProperty_CurrentHardwareSampleRate;
             let sample_rate: f64 = super::audio_session_get_property(id)?;
             (sample_rate * seconds as f64).round() as u32
         };
@@ -588,7 +576,7 @@ impl AudioUnit {
         let data_byte_size = buffer_frame_size * sample_bytes as u32 * n_channels;
         let mut data = vec![0u8; data_byte_size as usize];
         let mut buffer_capacity = data_byte_size as usize;
-        let audio_buffer = AudioBuffer {
+        let audio_buffer = sys::AudioBuffer {
             mDataByteSize: data_byte_size,
             mNumberChannels: n_channels,
             mData: data.as_mut_ptr() as *mut _,
@@ -597,7 +585,7 @@ impl AudioUnit {
         // TODO: This leaks the len & capacity fields, since only the buffer pointer is released
         mem::forget(data);
 
-        let audio_buffer_list = Box::new(AudioBufferList {
+        let audio_buffer_list = Box::new(sys::AudioBufferList {
             mNumberBuffers: 1,
             mBuffers: [audio_buffer],
         });
@@ -612,17 +600,17 @@ impl AudioUnit {
         // This allows us to take advantage of rust's type system and provide format-specific
         // `Args` types which can be checked at compile time.
         let audio_unit = self.instance;
-        let input_proc_fn = move |io_action_flags: NonNull<AudioUnitRenderActionFlags>,
-                                  in_time_stamp: NonNull<AudioTimeStamp>,
-                                  in_bus_number: u32,
-                                  in_number_frames: u32,
-                                  _io_data: *mut AudioBufferList|
-              -> OSStatus {
+        let input_proc_fn = move |io_action_flags: *mut sys::AudioUnitRenderActionFlags,
+                                  in_time_stamp: *const sys::AudioTimeStamp,
+                                  in_bus_number: sys::UInt32,
+                                  in_number_frames: sys::UInt32,
+                                  _io_data: *mut sys::AudioBufferList|
+              -> sys::OSStatus {
             // If the buffer size has changed, ensure the AudioBuffer is the correct size.
             if buffer_frame_size != in_number_frames {
                 unsafe {
                     // Retrieve the up-to-date stream format.
-                    let id = kAudioUnitProperty_StreamFormat;
+                    let id = sys::kAudioUnitProperty_StreamFormat;
                     let asbd =
                         match super::get_property(audio_unit, id, Scope::Output, Element::Input) {
                             Err(err) => return err.as_os_status(),
@@ -636,10 +624,10 @@ impl AudioUnit {
                     let n_channels = stream_format.channels;
                     let data_byte_size =
                         in_number_frames as usize * sample_bytes * n_channels as usize;
-                    let ptr = (*audio_buffer_list_ptr).mBuffers.as_ptr() as *mut AudioBuffer;
+                    let ptr = (*audio_buffer_list_ptr).mBuffers.as_ptr() as *mut sys::AudioBuffer;
                     let len = (*audio_buffer_list_ptr).mNumberBuffers as usize;
 
-                    let buffers: &mut [AudioBuffer] = slice::from_raw_parts_mut(ptr, len);
+                    let buffers: &mut [sys::AudioBuffer] = slice::from_raw_parts_mut(ptr, len);
                     let old_capacity = buffer_capacity;
                     for buffer in buffers {
                         let current_len = buffer.mDataByteSize as usize;
@@ -658,13 +646,13 @@ impl AudioUnit {
             }
 
             unsafe {
-                let status = AudioUnitRender(
+                let status = sys::AudioUnitRender(
                     audio_unit,
-                    io_action_flags.as_ptr(),
+                    io_action_flags,
                     in_time_stamp,
                     in_bus_number,
                     in_number_frames,
-                    NonNull::new(audio_buffer_list_ptr).unwrap(),
+                    audio_buffer_list_ptr,
                 );
                 if status != 0 {
                     return status;
@@ -673,10 +661,10 @@ impl AudioUnit {
 
             let args = unsafe {
                 let data = D::from_input_proc_args(in_number_frames, audio_buffer_list_ptr);
-                let flags = action_flags::Handle::from_ptr(io_action_flags.as_ptr());
+                let flags = action_flags::Handle::from_ptr(io_action_flags);
                 Args {
                     data,
-                    time_stamp: in_time_stamp.read(),
+                    time_stamp: *in_time_stamp,
                     flags,
                     bus_number: in_bus_number as u32,
                     num_frames: in_number_frames as usize,
@@ -699,13 +687,13 @@ impl AudioUnit {
         // within our AudioUnit's Drop implementation (otherwise it would leak).
         let input_proc_fn_wrapper_ptr = Box::into_raw(input_proc_fn_wrapper) as *mut c_void;
 
-        let render_callback = AURenderCallbackStruct {
+        let render_callback = sys::AURenderCallbackStruct {
             inputProc: Some(input_proc),
             inputProcRefCon: input_proc_fn_wrapper_ptr,
         };
 
         self.set_property(
-            kAudioOutputUnitProperty_SetInputCallback,
+            sys::kAudioOutputUnitProperty_SetInputCallback,
             Scope::Global,
             Element::Output,
             Some(&render_callback),
@@ -742,11 +730,11 @@ impl AudioUnit {
             } = input_callback;
             unsafe {
                 // Take ownership over the AudioBufferList in order to safely free it.
-                let buffer_list: Box<AudioBufferList> = Box::from_raw(buffer_list);
+                let buffer_list: Box<sys::AudioBufferList> = Box::from_raw(buffer_list);
                 // Free the allocated data from the individual audio buffers.
-                let ptr = buffer_list.mBuffers.as_ptr() as *const AudioBuffer;
+                let ptr = buffer_list.mBuffers.as_ptr() as *const sys::AudioBuffer;
                 let len = buffer_list.mNumberBuffers as usize;
-                let buffers: &[AudioBuffer] = slice::from_raw_parts(ptr, len);
+                let buffers: &[sys::AudioBuffer] = slice::from_raw_parts(ptr, len);
                 for &buffer in buffers {
                     let ptr = buffer.mData as *mut u8;
                     let len = buffer.mDataByteSize as usize;
@@ -763,20 +751,22 @@ impl AudioUnit {
 }
 
 /// Callback procedure that will be called each time our audio_unit requests audio.
-extern "C-unwind" fn input_proc(
-    in_ref_con: NonNull<c_void>,
-    io_action_flags: NonNull<AudioUnitRenderActionFlags>,
-    in_time_stamp: NonNull<AudioTimeStamp>,
-    in_bus_number: u32,
-    in_number_frames: u32,
-    io_data: *mut AudioBufferList,
-) -> OSStatus {
-    let wrapper = unsafe { in_ref_con.cast::<InputProcFnWrapper>().as_mut() };
-    (wrapper.callback)(
-        io_action_flags,
-        in_time_stamp,
-        in_bus_number,
-        in_number_frames,
-        io_data,
-    )
+extern "C" fn input_proc(
+    in_ref_con: *mut c_void,
+    io_action_flags: *mut sys::AudioUnitRenderActionFlags,
+    in_time_stamp: *const sys::AudioTimeStamp,
+    in_bus_number: sys::UInt32,
+    in_number_frames: sys::UInt32,
+    io_data: *mut sys::AudioBufferList,
+) -> sys::OSStatus {
+    let wrapper = in_ref_con as *mut InputProcFnWrapper;
+    unsafe {
+        (*(*wrapper).callback)(
+            io_action_flags,
+            in_time_stamp,
+            in_bus_number,
+            in_number_frames,
+            io_data,
+        )
+    }
 }
