@@ -148,7 +148,7 @@ impl Keyring {
             Err(err) => return Err(err.into()),
             Ok(mut file) => {
                 #[cfg(feature = "tracing")]
-                tracing::debug!("Keyring file found, loading it content");
+                tracing::debug!("Keyring file found, loading its content");
                 let mtime = file.metadata().await?.modified().ok();
 
                 let mut content = Vec::new();
@@ -160,41 +160,46 @@ impl Keyring {
             }
         };
 
-        if validate_items {
+        let key = if validate_items {
             let key = keyring.derive_key(&secret)?;
 
             let mut n_broken_items = 0;
             let mut n_valid_items = 0;
             for encrypted_item in &keyring.items {
-                if let Err(_err) = encrypted_item.clone().decrypt(&key) {
-                    #[cfg(feature = "tracing")]
-                    tracing::warn!("Failed to decrypt item, incorrect secret?: {_err}");
+                if encrypted_item.clone().decrypt(&key).is_err() {
                     n_broken_items += 1;
                 } else {
                     n_valid_items += 1;
                 }
             }
-
-            if n_broken_items > n_valid_items {
+            if n_valid_items == 0 && n_broken_items != 0 {
+                #[cfg(feature = "tracing")]
+                tracing::error!("Keyring cannot be decrypted. Invalid secret.");
+                return Err(Error::IncorrectSecret);
+            } else if n_broken_items > n_valid_items {
                 #[cfg(feature = "tracing")]
                 {
                     tracing::warn!(
                         "The file contains {n_broken_items} broken items and {n_valid_items} valid ones."
                     );
-                    tracing::info!("Please switch to Keyring::load_unchecked to load the keyring without the secret validation");
-                    tracing::info!("Keyring::delete_broken_items can be used to remove them");
+                    tracing::info!(
+                        "Please switch to `Keyring::load_unchecked` to load the keyring without the secret validation.
+                        `Keyring::delete_broken_items` can be used to remove them or alternatively with `oo7-cli --repair`."
+                    );
                 }
                 return Err(Error::IncorrectSecret);
             }
 
-            drop(key);
-        }
+            Some(Arc::new(key))
+        } else {
+            None
+        };
 
         Ok(Self {
             keyring: Arc::new(RwLock::new(keyring)),
             path: Some(path.as_ref().to_path_buf()),
             mtime: Mutex::new(mtime),
-            key: Default::default(),
+            key: Mutex::new(key),
             secret: Mutex::new(Arc::new(secret)),
         })
     }
@@ -668,7 +673,7 @@ mod tests {
         assert_eq!(items[0].label(), "foo");
         assert_eq!(items[0].secret(), Secret::blob("foo"));
         let attributes = items[0].attributes();
-        assert_eq!(attributes.len(), 1);
+        assert_eq!(attributes.len(), 2);
         assert_eq!(
             attributes
                 .get(crate::XDG_SCHEMA_ATTRIBUTE)
@@ -691,7 +696,9 @@ mod tests {
             .join("legacy.keyring");
         fs::copy(&fixture_path, &v0_dir.join("default.keyring")).await?;
 
-        std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        unsafe {
+            std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        }
 
         assert!(!v1_dir.join("default.keyring").exists());
 
@@ -718,7 +725,9 @@ mod tests {
             .join("default.keyring");
         fs::copy(&fixture_path, &v0_dir.join("default.keyring")).await?;
 
-        std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        unsafe {
+            std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        }
 
         let secret = Secret::blob("test");
         let keyring = Keyring::open("default", secret).await?;
@@ -745,7 +754,9 @@ mod tests {
             .join("default.keyring");
         fs::copy(&fixture_path, &v1_dir.join("default.keyring")).await?;
 
-        std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        unsafe {
+            std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        }
 
         let secret = Secret::blob("wrong");
         let keyring = Keyring::open("default", secret).await;
@@ -773,7 +784,9 @@ mod tests {
             .join("default.keyring");
         fs::copy(&fixture_path, &v1_dir.join("default.keyring")).await?;
 
-        std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        unsafe {
+            std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        }
 
         let secret = Secret::blob("test");
         let keyring = Keyring::open("default", secret).await?;
@@ -795,7 +808,9 @@ mod tests {
         let v1_dir = v0_dir.join("v1");
         fs::create_dir_all(&v1_dir).await?;
 
-        std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        unsafe {
+            std::env::set_var("XDG_DATA_HOME", data_dir.path());
+        }
 
         let secret = Secret::blob("test");
         let keyring = Keyring::open("default", secret).await?;
@@ -854,9 +869,11 @@ mod tests {
             .await?;
         drop(keyring);
 
-        assert!(Keyring::load(&keyring_path, Secret::blob("wrong_password"))
-            .await
-            .is_err());
+        assert!(
+            Keyring::load(&keyring_path, Secret::blob("wrong_password"))
+                .await
+                .is_err()
+        );
 
         let keyring = Keyring::load(&keyring_path, Secret::blob("test")).await?;
         keyring
@@ -913,4 +930,44 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn content_type() -> Result<(), Error> {
+        use crate::secret::ContentType;
+
+        let keyring = Keyring::temporary(Secret::blob("test_password")).await?;
+
+        // Add items with different MIME types
+        keyring
+            .create_item(
+                "Text",
+                &HashMap::from([("type", "text")]),
+                Secret::text("Hello, World!"),
+                false,
+            )
+            .await?;
+
+        keyring
+            .create_item(
+                "Password",
+                &HashMap::from([("type", "password")]),
+                Secret::blob("super_secret_password"),
+                false,
+            )
+            .await?;
+
+        let items = keyring
+            .search_items(&HashMap::from([("type", "text")]))
+            .await?;
+        assert_eq!(items.len(), 1);
+        assert_eq!(items[0].secret().content_type(), ContentType::Text);
+
+        let items = keyring
+            .search_items(&HashMap::from([("type", "password")]))
+            .await?;
+        assert_eq!(items.len(), 1);
+        assert_eq!(items[0].secret().content_type(), ContentType::Blob);
+
+        Ok(())
+    }
 }
