@@ -1,11 +1,9 @@
 //! Contains high-level interface for an events-based XML emitter.
 
-use std::borrow::Cow;
 use std::io::Write;
-use std::result::Result as StdResult;
 
 use crate::encoding::UTF8_BOM;
-use crate::errors::{Error, Result};
+use crate::errors::Result;
 use crate::events::{attributes::Attribute, BytesCData, BytesStart, BytesText, Event};
 
 #[cfg(feature = "async-tokio")]
@@ -28,7 +26,7 @@ use {crate::de::DeError, serde::Serialize};
 ///
 /// let xml = r#"<this_tag k1="v1" k2="v2"><child>text</child></this_tag>"#;
 /// let mut reader = Reader::from_str(xml);
-/// reader.config_mut().trim_text(true);
+/// reader.trim_text(true);
 /// let mut writer = Writer::new(Cursor::new(Vec::new()));
 /// loop {
 ///     match reader.read_event() {
@@ -99,62 +97,6 @@ impl<W> Writer<W> {
     pub fn get_ref(&self) -> &W {
         &self.writer
     }
-
-    /// Provides a simple, high-level API for writing XML elements.
-    ///
-    /// Returns an [`ElementWriter`] that simplifies setting attributes and writing
-    /// content inside the element.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # use quick_xml::Result;
-    /// # fn main() -> Result<()> {
-    /// use quick_xml::events::{BytesStart, BytesText, Event};
-    /// use quick_xml::writer::Writer;
-    /// use quick_xml::Error;
-    /// use std::io::Cursor;
-    ///
-    /// let mut writer = Writer::new(Cursor::new(Vec::new()));
-    ///
-    /// // writes <tag attr1="value1"/>
-    /// writer.create_element("tag")
-    ///     .with_attribute(("attr1", "value1"))  // chain `with_attribute()` calls to add many attributes
-    ///     .write_empty()?;
-    ///
-    /// // writes <tag attr1="value1" attr2="value2">with some text inside</tag>
-    /// writer.create_element("tag")
-    ///     .with_attributes(vec![("attr1", "value1"), ("attr2", "value2")].into_iter())  // or add attributes from an iterator
-    ///     .write_text_content(BytesText::new("with some text inside"))?;
-    ///
-    /// // writes <tag><fruit quantity="0">apple</fruit><fruit quantity="1">orange</fruit></tag>
-    /// writer.create_element("tag")
-    ///     // We need to provide error type, because it is not named somewhere explicitly
-    ///     .write_inner_content::<_, Error>(|writer| {
-    ///         let fruits = ["apple", "orange"];
-    ///         for (quant, item) in fruits.iter().enumerate() {
-    ///             writer
-    ///                 .create_element("fruit")
-    ///                 .with_attribute(("quantity", quant.to_string().as_str()))
-    ///                 .write_text_content(BytesText::new(item))?;
-    ///         }
-    ///         Ok(())
-    ///     })?;
-    /// # Ok(())
-    /// # }
-    /// ```
-    #[must_use]
-    pub fn create_element<'a, N>(&'a mut self, name: N) -> ElementWriter<W>
-    where
-        N: Into<Cow<'a, str>>,
-    {
-        ElementWriter {
-            writer: self,
-            start_tag: BytesStart::new(name),
-            state: AttributeIndent::NoneAttributesWritten,
-            spaces: Vec::new(),
-        }
-    }
 }
 
 impl<W: Write> Writer<W> {
@@ -271,6 +213,59 @@ impl<W: Write> Writer<W> {
         Ok(())
     }
 
+    /// Provides a simple, high-level API for writing XML elements.
+    ///
+    /// Returns an [`ElementWriter`] that simplifies setting attributes and writing
+    /// content inside the element.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use quick_xml::Result;
+    /// # fn main() -> Result<()> {
+    /// use quick_xml::events::{BytesStart, BytesText, Event};
+    /// use quick_xml::writer::Writer;
+    /// use quick_xml::Error;
+    /// use std::io::Cursor;
+    ///
+    /// let mut writer = Writer::new(Cursor::new(Vec::new()));
+    ///
+    /// // writes <tag attr1="value1"/>
+    /// writer.create_element("tag")
+    ///     .with_attribute(("attr1", "value1"))  // chain `with_attribute()` calls to add many attributes
+    ///     .write_empty()?;
+    ///
+    /// // writes <tag attr1="value1" attr2="value2">with some text inside</tag>
+    /// writer.create_element("tag")
+    ///     .with_attributes(vec![("attr1", "value1"), ("attr2", "value2")].into_iter())  // or add attributes from an iterator
+    ///     .write_text_content(BytesText::new("with some text inside"))?;
+    ///
+    /// // writes <tag><fruit quantity="0">apple</fruit><fruit quantity="1">orange</fruit></tag>
+    /// writer.create_element("tag")
+    ///     .write_inner_content(|writer| {
+    ///         let fruits = ["apple", "orange"];
+    ///         for (quant, item) in fruits.iter().enumerate() {
+    ///             writer
+    ///                 .create_element("fruit")
+    ///                 .with_attribute(("quantity", quant.to_string().as_str()))
+    ///                 .write_text_content(BytesText::new(item))?;
+    ///         }
+    ///         Ok(())
+    ///     })?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    #[must_use]
+    pub fn create_element<'a, N>(&'a mut self, name: &'a N) -> ElementWriter<W>
+    where
+        N: 'a + AsRef<str> + ?Sized,
+    {
+        ElementWriter {
+            writer: self,
+            start_tag: BytesStart::new(name.as_ref()),
+        }
+    }
+
     /// Write an arbitrary serializable type
     ///
     /// Note: If you are attempting to write XML in a non-UTF-8 encoding, this may not
@@ -338,57 +333,20 @@ impl<W: Write> Writer<W> {
     }
 }
 
-/// Track indent inside elements state
-///
-/// ```mermaid
-/// stateDiagram-v2
-///     [*] --> NoneAttributesWritten
-///     NoneAttributesWritten --> Spaces : .with_attribute()
-///     NoneAttributesWritten --> WriteConfigured : .new_line()
-///
-///     Spaces --> Spaces : .with_attribute()
-///     Spaces --> WriteSpaces : .new_line()
-///
-///     WriteSpaces --> Spaces : .with_attribute()
-///     WriteSpaces --> WriteSpaces : .new_line()
-///
-///     Configured --> Configured : .with_attribute()
-///     Configured --> WriteConfigured : .new_line()
-///
-///     WriteConfigured --> Configured : .with_attribute()
-///     WriteConfigured --> WriteConfigured : .new_line()
-/// ```
-#[derive(Debug)]
-enum AttributeIndent {
-    /// Initial state. `ElementWriter` was just created and no attributes written yet
-    NoneAttributesWritten,
-    /// Write specified count of spaces to indent before writing attribute in `with_attribute()`
-    WriteSpaces(usize),
-    /// Keep space indent that should be used if `new_line()` would be called
-    Spaces(usize),
-    /// Write specified count of indent characters before writing attribute in `with_attribute()`
-    WriteConfigured(usize),
-    /// Keep indent that should be used if `new_line()` would be called
-    Configured(usize),
-}
-
 /// A struct to write an element. Contains methods to add attributes and inner
 /// elements to the element
-pub struct ElementWriter<'a, W> {
+pub struct ElementWriter<'a, W: Write> {
     writer: &'a mut Writer<W>,
     start_tag: BytesStart<'a>,
-    state: AttributeIndent,
-    /// Contains spaces used to write space indents of attributes
-    spaces: Vec<u8>,
 }
 
-impl<'a, W> ElementWriter<'a, W> {
+impl<'a, W: Write> ElementWriter<'a, W> {
     /// Adds an attribute to this element.
     pub fn with_attribute<'b, I>(mut self, attr: I) -> Self
     where
         I: Into<Attribute<'b>>,
     {
-        self.write_attr(attr.into());
+        self.start_tag.push_attribute(attr);
         self
     }
 
@@ -400,136 +358,10 @@ impl<'a, W> ElementWriter<'a, W> {
         I: IntoIterator,
         I::Item: Into<Attribute<'b>>,
     {
-        let mut iter = attributes.into_iter();
-        if let Some(attr) = iter.next() {
-            self.write_attr(attr.into());
-            self.start_tag.extend_attributes(iter);
-        }
+        self.start_tag.extend_attributes(attributes);
         self
     }
 
-    /// Push a new line inside an element between attributes. Note, that this
-    /// method does nothing if [`Writer`] was created without indentation support.
-    ///
-    /// # Examples
-    ///
-    /// The following code
-    ///
-    /// ```
-    /// # use quick_xml::writer::Writer;
-    /// let mut buffer = Vec::new();
-    /// let mut writer = Writer::new_with_indent(&mut buffer, b' ', 2);
-    /// writer
-    ///   .create_element("element")
-    ///     //.new_line() (1)
-    ///     .with_attribute(("first", "1"))
-    ///     .with_attribute(("second", "2"))
-    ///     .new_line()
-    ///     .with_attributes([
-    ///         ("third", "3"),
-    ///         ("fourth", "4"),
-    ///     ])
-    ///     //.new_line() (2)
-    ///     .write_empty();
-    /// ```
-    /// will produce the following XMLs:
-    /// ```xml
-    /// <!-- result of the code above. Spaces always is used -->
-    /// <element first="1" second="2"
-    ///          third="3" fourth="4"/>
-    ///
-    /// <!-- if uncomment only (1) - indent depends on indentation
-    ///      settings - 2 spaces here -->
-    /// <element
-    ///   first="1" second="2"
-    ///   third="3" fourth="4"/>
-    ///
-    /// <!-- if uncomment only (2). Spaces always is used  -->
-    /// <element first="1" second="2"
-    ///          third="3" fourth="4"
-    /// />
-    /// ```
-    pub fn new_line(mut self) -> Self {
-        if let Some(i) = self.writer.indent.as_mut() {
-            match self.state {
-                // .new_line() called just after .create_element().
-                // Use element indent to additionally indent attributes
-                AttributeIndent::NoneAttributesWritten => {
-                    self.state = AttributeIndent::WriteConfigured(i.indent_size)
-                }
-
-                AttributeIndent::WriteSpaces(_) => {}
-                // .new_line() called when .with_attribute() was called at least once.
-                // The spaces should be used to indent
-                // Plan saved indent
-                AttributeIndent::Spaces(indent) => {
-                    self.state = AttributeIndent::WriteSpaces(indent)
-                }
-
-                AttributeIndent::WriteConfigured(_) => {}
-                // .new_line() called when .with_attribute() was called at least once.
-                // The configured indent characters should be used to indent
-                // Plan saved indent
-                AttributeIndent::Configured(indent) => {
-                    self.state = AttributeIndent::WriteConfigured(indent)
-                }
-            }
-            self.start_tag.push_newline();
-        };
-        self
-    }
-
-    /// Writes attribute and maintain indentation state
-    fn write_attr<'b>(&mut self, attr: Attribute<'b>) {
-        if let Some(i) = self.writer.indent.as_mut() {
-            // Save the indent that we should use next time when .new_line() be called
-            self.state = match self.state {
-                // Neither .new_line() or .with_attribute() yet called
-                // If newline inside attributes will be requested, we should indent them
-                // by the length of tag name and +1 for `<` and +1 for one space
-                AttributeIndent::NoneAttributesWritten => {
-                    self.start_tag.push_attribute(attr);
-                    AttributeIndent::Spaces(self.start_tag.name().as_ref().len() + 2)
-                }
-
-                // Indent was requested by previous call to .new_line(), write it
-                // New line was already written
-                AttributeIndent::WriteSpaces(indent) => {
-                    if self.spaces.len() < indent {
-                        self.spaces.resize(indent, b' ');
-                    }
-                    self.start_tag.push_indent(&self.spaces[..indent]);
-                    self.start_tag.push_attr(attr.into());
-                    AttributeIndent::Spaces(indent)
-                }
-                // .new_line() was not called, but .with_attribute() was.
-                // use the previously calculated indent
-                AttributeIndent::Spaces(indent) => {
-                    self.start_tag.push_attribute(attr);
-                    AttributeIndent::Spaces(indent)
-                }
-
-                // Indent was requested by previous call to .new_line(), write it
-                // New line was already written
-                AttributeIndent::WriteConfigured(indent) => {
-                    self.start_tag.push_indent(i.additional(indent));
-                    self.start_tag.push_attr(attr.into());
-                    AttributeIndent::Configured(indent)
-                }
-                // .new_line() was not called, but .with_attribute() was.
-                // use the previously calculated indent
-                AttributeIndent::Configured(indent) => {
-                    self.start_tag.push_attribute(attr);
-                    AttributeIndent::Configured(indent)
-                }
-            };
-        } else {
-            self.start_tag.push_attribute(attr);
-        }
-    }
-}
-
-impl<'a, W: Write> ElementWriter<'a, W> {
     /// Write some text inside the current element.
     pub fn write_text_content(self, text: BytesText) -> Result<&'a mut Writer<W>> {
         self.writer
@@ -567,10 +399,9 @@ impl<'a, W: Write> ElementWriter<'a, W> {
     }
 
     /// Create a new scope for writing XML inside the current element.
-    pub fn write_inner_content<F, E>(self, closure: F) -> StdResult<&'a mut Writer<W>, E>
+    pub fn write_inner_content<F>(self, closure: F) -> Result<&'a mut Writer<W>>
     where
-        F: FnOnce(&mut Writer<W>) -> StdResult<(), E>,
-        E: From<Error>,
+        F: FnOnce(&mut Writer<W>) -> Result<()>,
     {
         self.writer
             .write_event(Event::Start(self.start_tag.borrow()))?;
@@ -622,7 +453,10 @@ impl Indentation {
     /// Increase indentation by one level
     pub fn grow(&mut self) {
         self.current_indent_len += self.indent_size;
-        self.ensure(self.current_indent_len);
+        if self.current_indent_len > self.indents.len() {
+            self.indents
+                .resize(self.current_indent_len, self.indent_char);
+        }
     }
 
     /// Decrease indentation by one level. Do nothing, if level already zero
@@ -634,19 +468,6 @@ impl Indentation {
     pub fn current(&self) -> &[u8] {
         &self.indents[..self.current_indent_len]
     }
-
-    /// Returns indent with current indent plus additional indent
-    pub fn additional(&mut self, additional_indent: usize) -> &[u8] {
-        let new_len = self.current_indent_len + additional_indent;
-        self.ensure(new_len);
-        &self.indents[..new_len]
-    }
-
-    fn ensure(&mut self, new_len: usize) {
-        if self.indents.len() < new_len {
-            self.indents.resize(new_len, self.indent_char);
-        }
-    }
 }
 
 #[cfg(test)]
@@ -925,7 +746,7 @@ mod indentation {
             .create_element("outer")
             .with_attribute(("attr1", "value1"))
             .with_attribute(("attr2", "value2"))
-            .write_inner_content::<_, Error>(|writer| {
+            .write_inner_content(|writer| {
                 let fruits = ["apple", "orange", "banana"];
                 for (quant, item) in fruits.iter().enumerate() {
                     writer
@@ -936,7 +757,8 @@ mod indentation {
                 writer
                     .create_element("inner")
                     .write_inner_content(|writer| {
-                        writer.create_element("empty").write_empty().map(|_| ())
+                        writer.create_element("empty").write_empty()?;
+                        Ok(())
                     })?;
 
                 Ok(())
@@ -955,251 +777,4 @@ mod indentation {
 </outer>"#
         );
     }
-
-    mod in_attributes {
-        use super::*;
-        use pretty_assertions::assert_eq;
-
-        #[test]
-        fn newline_first() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .with_attribute(("first", "1"))
-                .with_attribute(("second", "2"))
-                .new_line()
-                .with_attribute(("third", "3"))
-                .with_attribute(("fourth", "4"))
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                "<element\
-                    \n_first=\"1\" second=\"2\"\
-                    \n_third=\"3\" fourth=\"4\"/>"
-            );
-        }
-
-        #[test]
-        fn newline_inside() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .with_attribute(("first", "1"))
-                .with_attribute(("second", "2"))
-                .new_line()
-                .with_attribute(("third", "3"))
-                .with_attribute(("fourth", "4"))
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                "<element first=\"1\" second=\"2\"\
-                \n         third=\"3\" fourth=\"4\"/>"
-            );
-        }
-
-        #[test]
-        fn newline_last() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .with_attribute(("first", "1"))
-                .with_attribute(("second", "2"))
-                .new_line()
-                .with_attribute(("third", "3"))
-                .with_attribute(("fourth", "4"))
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            writer
-                .create_element("element")
-                .with_attribute(("first", "1"))
-                .with_attribute(("second", "2"))
-                .new_line()
-                .with_attribute(("third", "3"))
-                .with_attribute(("fourth", "4"))
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                "<element\
-                    \n_first=\"1\" second=\"2\"\
-                    \n_third=\"3\" fourth=\"4\"\
-                \n/>\
-                \n<element first=\"1\" second=\"2\"\
-                \n         third=\"3\" fourth=\"4\"\
-                \n/>"
-            );
-        }
-
-        #[test]
-        fn newline_twice() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            writer
-                .create_element("element")
-                .with_attribute(("first", "1"))
-                .new_line()
-                .new_line()
-                .with_attribute(("second", "2"))
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                r#"<element
-
-/>
-<element first="1"
-
-         second="2"/>"#
-            );
-        }
-
-        #[test]
-        fn without_indent() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new(&mut buffer);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            writer
-                .create_element("element")
-                .with_attribute(("first", "1"))
-                .new_line()
-                .new_line()
-                .with_attribute(("second", "2"))
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                r#"<element/><element first="1" second="2"/>"#
-            );
-        }
-
-        #[test]
-        fn long_element_name() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b't', 1);
-
-            writer
-                .create_element(String::from("x").repeat(128).as_str())
-                .with_attribute(("first", "1"))
-                .new_line()
-                .with_attribute(("second", "2"))
-                .write_empty()
-                .expect("Problem with indentation reference");
-        }
-    }
-
-    mod in_attributes_multi {
-        use super::*;
-        use pretty_assertions::assert_eq;
-
-        #[test]
-        fn newline_first() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .with_attributes([("first", "1"), ("second", "2")])
-                .new_line()
-                .with_attributes([("third", "3"), ("fourth", "4")])
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                "<element\
-                    \n_first=\"1\" second=\"2\"\
-                    \n_third=\"3\" fourth=\"4\"/>"
-            );
-        }
-
-        #[test]
-        fn newline_inside() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .with_attributes([("first", "1"), ("second", "2")])
-                .new_line()
-                .with_attributes([("third", "3"), ("fourth", "4")])
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                r#"<element first="1" second="2"
-         third="3" fourth="4"/>"#
-            );
-        }
-
-        #[test]
-        fn newline_last() {
-            let mut buffer = Vec::new();
-            let mut writer = Writer::new_with_indent(&mut buffer, b'_', 1);
-
-            writer
-                .create_element("element")
-                .new_line()
-                .with_attributes([("first", "1"), ("second", "2")])
-                .new_line()
-                .with_attributes([("third", "3"), ("fourth", "4")])
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            writer
-                .create_element("element")
-                .with_attributes([("first", "1"), ("second", "2")])
-                .new_line()
-                .with_attributes([("third", "3"), ("fourth", "4")])
-                .new_line()
-                .write_empty()
-                .expect("write tag failed");
-
-            assert_eq!(
-                std::str::from_utf8(&buffer).unwrap(),
-                "<element\
-                    \n_first=\"1\" second=\"2\"\
-                    \n_third=\"3\" fourth=\"4\"\
-                \n/>\
-                \n<element first=\"1\" second=\"2\"\
-                \n         third=\"3\" fourth=\"4\"\
-                \n/>"
-            );
-        }
-    }
 }
