@@ -57,9 +57,36 @@ impl<'a> GlyphVariationDataHeader<'a> {
 }
 
 impl<'a> Gvar<'a> {
-    pub fn data_for_gid(&self, gid: GlyphId) -> Result<FontData<'a>, ReadError> {
+    /// Return the raw data for this gid.
+    ///
+    /// If there is no variation data for the glyph, returns `Ok(None)`.
+    pub fn data_for_gid(&self, gid: GlyphId) -> Result<Option<FontData<'a>>, ReadError> {
         let range = self.data_range_for_gid(gid)?;
-        self.data.slice(range).ok_or(ReadError::OutOfBounds)
+        if range.is_empty() {
+            return Ok(None);
+        }
+        match self.data.slice(range) {
+            Some(data) => Ok(Some(data)),
+            None => Err(ReadError::OutOfBounds),
+        }
+    }
+
+    pub fn glyph_variation_data_for_range(
+        &self,
+        offset_range: Range<usize>,
+    ) -> Result<FontData<'a>, ReadError> {
+        let base = self.glyph_variation_data_array_offset() as usize;
+        let start = base
+            .checked_add(offset_range.start)
+            .ok_or(ReadError::OutOfBounds)?;
+        let end = base
+            .checked_add(offset_range.end)
+            .ok_or(ReadError::OutOfBounds)?;
+        self.data.slice(start..end).ok_or(ReadError::OutOfBounds)
+    }
+
+    pub fn as_bytes(&self) -> &[u8] {
+        self.data.as_bytes()
     }
 
     fn data_range_for_gid(&self, gid: GlyphId) -> Result<Range<usize>, ReadError> {
@@ -76,15 +103,22 @@ impl<'a> Gvar<'a> {
     }
 
     /// Get the variation data for a specific glyph.
-    pub fn glyph_variation_data(&self, gid: GlyphId) -> Result<GlyphVariationData<'a>, ReadError> {
+    ///
+    /// Returns `Ok(None)` if there is no variation data for this glyph, and
+    /// returns an error if there is data but it is malformed.
+    pub fn glyph_variation_data(
+        &self,
+        gid: GlyphId,
+    ) -> Result<Option<GlyphVariationData<'a>>, ReadError> {
         let shared_tuples = self.shared_tuples()?;
         let axis_count = self.axis_count();
         let data = self.data_for_gid(gid)?;
-        GlyphVariationData::new(data, axis_count, shared_tuples)
+        data.map(|data| GlyphVariationData::new(data, axis_count, shared_tuples))
+            .transpose()
     }
 
     /// Returns the phantom point deltas for the given variation coordinates
-    /// and glyph identifier.
+    /// and glyph identifier, if variation data exists for the glyph.
     ///
     /// The resulting array will contain four deltas:
     /// `[left, right, top, bottom]`.
@@ -94,10 +128,10 @@ impl<'a> Gvar<'a> {
         loca: &Loca,
         coords: &[F2Dot14],
         glyph_id: GlyphId,
-    ) -> Result<[Point<Fixed>; 4], ReadError> {
+    ) -> Result<Option<[Point<Fixed>; 4]>, ReadError> {
         // For any given glyph, there's only one outline that contributes to
         // metrics deltas (via "phantom points"). For simple glyphs, that is
-        // the glyph itself. For composite glyphs, it is the first component
+        // the glyph itself. For composite glyphs, it is the last component
         // in the tree that has the USE_MY_METRICS flag set or, if there are
         // none, the composite glyph itself.
         //
@@ -108,7 +142,9 @@ impl<'a> Gvar<'a> {
         let (glyph_id, point_count) = find_glyph_and_point_count(glyf, loca, glyph_id, 0)?;
         let mut phantom_deltas = [Point::default(); 4];
         let phantom_range = point_count..point_count + 4;
-        let var_data = self.glyph_variation_data(glyph_id)?;
+        let Some(var_data) = self.glyph_variation_data(glyph_id)? else {
+            return Ok(None);
+        };
         // Note that phantom points can never belong to a contour so we don't have
         // to handle the IUP case here.
         for (tuple, scalar) in var_data.active_tuples_at(coords) {
@@ -119,7 +155,7 @@ impl<'a> Gvar<'a> {
                 }
             }
         }
-        Ok(phantom_deltas)
+        Ok(Some(phantom_deltas))
     }
 }
 
@@ -147,7 +183,7 @@ impl<'a> GlyphVariationData<'a> {
         Ok(GlyphVariationData {
             tuple_count: count,
             axis_count,
-            shared_tuples: Some(shared_tuples),
+            shared_tuples: Some(shared_tuples.tuples()),
             shared_point_numbers,
             header_data,
             serialized_data,
@@ -223,34 +259,38 @@ fn find_glyph_and_point_count(
             Ok((glyph_id, simple.num_points()))
         }
         Glyph::Composite(composite) => {
-            // For composite glyphs, if one of the components has the
-            // USE_MY_METRICS flag set, recurse into the glyph referenced
-            // by that component. Otherwise, return the composite glyph
-            // itself and the number of components as the point count.
-            let mut count = 0;
-            for component in composite.components() {
-                count += 1;
-                if component
-                    .flags
-                    .contains(CompositeGlyphFlags::USE_MY_METRICS)
-                {
-                    return find_glyph_and_point_count(
-                        glyf,
-                        loca,
-                        component.glyph.into(),
-                        recurse_depth + 1,
-                    );
-                }
+            // For composite glyphs, recurse into the glyph referenced by the
+            // *last* component that has the USE_MY_METRICS flag set.
+            // Otherwise, return the composite glyph itself and the number of
+            // components as the point count.
+            // https://learn.microsoft.com/en-us/typography/opentype/spec/gvar#point-numbers-and-processing-for-composite-glyphs
+            let (count, inherit_metrics) = composite.component_glyphs_and_flags().fold(
+                (0, None),
+                |(count, inherit_metrics), (component_id, flags)| {
+                    let has_flag = flags.contains(CompositeGlyphFlags::USE_MY_METRICS);
+                    let preferred = has_flag.then_some(component_id).or(inherit_metrics);
+
+                    (count + 1, preferred)
+                },
+            );
+
+            if let Some(component) = inherit_metrics {
+                find_glyph_and_point_count(glyf, loca, component.into(), recurse_depth + 1)
+            } else {
+                Ok((glyph_id, count))
             }
-            Ok((glyph_id, count))
         }
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use std::collections::HashMap;
+
+    use font_test_data::bebuffer::BeBuffer;
+
     use super::*;
-    use crate::{test_helpers::BeBuffer, FontRef, TableProvider};
+    use crate::{FontRef, TableProvider};
 
     // Shared tuples in the 'gvar' table of the Skia font, as printed
     // in Apple's TrueType specification.
@@ -377,7 +417,7 @@ mod tests {
             .unwrap()
             .gvar()
             .unwrap();
-        let a_glyph_var = gvar.glyph_variation_data(GlyphId::new(1)).unwrap();
+        let a_glyph_var = gvar.glyph_variation_data(GlyphId::new(1)).unwrap().unwrap();
         assert_eq!(a_glyph_var.axis_count, 1);
         let mut tuples = a_glyph_var.tuples();
         let tup1 = tuples.next().unwrap();
@@ -411,7 +451,7 @@ mod tests {
             .unwrap()
             .gvar()
             .unwrap();
-        let agrave_glyph_var = gvar.glyph_variation_data(GlyphId::new(2)).unwrap();
+        let agrave_glyph_var = gvar.glyph_variation_data(GlyphId::new(2)).unwrap().unwrap();
         let mut tuples = agrave_glyph_var.tuples();
         let tup1 = tuples.next().unwrap();
         assert_eq!(
@@ -435,7 +475,7 @@ mod tests {
             .unwrap()
             .gvar()
             .unwrap();
-        let grave_glyph_var = gvar.glyph_variation_data(GlyphId::new(3)).unwrap();
+        let grave_glyph_var = gvar.glyph_variation_data(GlyphId::new(3)).unwrap().unwrap();
         let mut tuples = grave_glyph_var.tuples();
         let tup1 = tuples.next().unwrap();
         let tup2 = tuples.next().unwrap();
@@ -503,6 +543,7 @@ mod tests {
             .map(|coord| F2Dot14::from_f32(*coord))
             .collect::<Vec<_>>();
         gvar.phantom_point_deltas(&glyf, &loca, &coords, glyph_id)
+            .unwrap()
             .unwrap()
             .map(|delta| delta.map(Fixed::to_f32))
             .map(|p| (p.x, p.y))
@@ -529,8 +570,46 @@ mod tests {
         buf = buf.push(u32::MAX - 10);
         // two 32-bit entries that overflow when added to the above offset
         buf = buf.push(0u32).push(11u32);
-        let gvar = Gvar::read(buf.font_data()).unwrap();
+        let gvar = Gvar::read(buf.data().into()).unwrap();
         // don't panic with overflow!
         let _ = gvar.data_range_for_gid(GlyphId::new(0));
     }
+
+    // Test that we select the correct component to derive metrics from,
+    // considering ambiguity. Only covers the shallow case.
+    #[test]
+    fn follow_use_my_metrics() {
+        // Load the font and required tables
+        let font = FontRef::new(font_test_data::gvar::USE_MY_METRICS).unwrap();
+        let glyf = font.glyf().unwrap();
+        let loca = font.loca(None).unwrap();
+        let post = font.post().unwrap();
+
+        // Grab names for test quality-of-life and legibility
+        let gids = (0..post.num_glyphs().unwrap())
+            .map(GlyphId16::new)
+            .map(|gid| (post.glyph_name(gid).unwrap(), gid))
+            .collect::<HashMap<_, _>>();
+
+        // Test various cases
+        let (source, _) =
+            find_glyph_and_point_count(&glyf, &loca, gids["neither"].into(), 5).unwrap();
+        assert_eq!(
+            source, gids["neither"],
+            "a composite without any USE_MY_METRICS components should use its own metrics"
+        );
+
+        let (source, _) =
+            find_glyph_and_point_count(&glyf, &loca, gids["firstonly"].into(), 5).unwrap();
+        assert_eq!(
+            source, gids["first"],
+            "a composite with a single USE_MY_METRICS component should use that component's metrics"
+        );
+
+        let (source, _) = find_glyph_and_point_count(&glyf, &loca, gids["both"].into(), 5).unwrap();
+        assert_eq!(
+            source, gids["second"],
+            "a composite with multiple USE_MY_METRICS components should use the last flagged component's metrics"
+         );
+    }
 }
