@@ -1,13 +1,13 @@
 use alloc::{string::String, vec::Vec};
-use core::cmp::min;
 use core::convert::TryFrom;
+use core::cmp::min;
+
 use ttf_parser::GlyphId;
 
-use super::buffer::glyph_flag::{SAFE_TO_INSERT_TATWEEL, UNSAFE_TO_BREAK, UNSAFE_TO_CONCAT};
-use super::face::hb_glyph_extents_t;
+use super::buffer::glyph_flag::{UNSAFE_TO_BREAK, UNSAFE_TO_CONCAT};
+use super::face::GlyphExtents;
 use super::unicode::{CharExt, GeneralCategoryExt};
 use super::{hb_font_t, hb_mask_t};
-use crate::hb::set_digest::{hb_set_digest_ext, hb_set_digest_t};
 use crate::{script, BufferClusterLevel, BufferFlags, Direction, Language, Script, SerializeFlags};
 
 const CONTEXT_LENGTH: usize = 5;
@@ -49,16 +49,16 @@ pub mod glyph_flag {
     /// line-break position, in the following way:
     ///
     /// 1. Iterate back from the line-break
-    ///    position until the first cluster
-    ///    start position that is NOT unsafe-to-concat,
+    /// position until the first cluster
+    /// start position that is NOT unsafe-to-concat,
     /// 2. shape the segment from there till the
-    ///    end of line, 3. check whether the resulting
-    ///    glyph-run also is clear of the unsafe-to-concat
-    ///    at its start-of-text position; if it is, just
-    ///    splice it into place and the line is shaped;
-    ///    If not, move on to a position further back that
-    ///    is clear of unsafe-to-concat and retry from
-    ///    there, and repeat.
+    /// end of line, 3. check whether the resulting
+    /// glyph-run also is clear of the unsafe-to-concat
+    /// at its start-of-text position; if it is, just
+    /// splice it into place and the line is shaped;
+    /// If not, move on to a position further back that
+    /// is clear of unsafe-to-concat and retry from
+    /// there, and repeat.
     ///
     /// At the start of next line a similar
     /// algorithm can be implemented.
@@ -87,18 +87,12 @@ pub mod glyph_flag {
     ///
     /// The UNSAFE_TO_BREAK flag will always imply this flag.
     /// To use this flag, you must enable the buffer flag
-    /// PRODUCE_UNSAFE_TO_CONCAT during shaping, otherwise
+    ///	PRODUCE_UNSAFE_TO_CONCAT during shaping, otherwise
     /// the buffer flag will not be reliably produced.
     pub const UNSAFE_TO_CONCAT: u32 = 0x00000002;
 
-    /// In scripts that use elongation (Arabic,
-    /// Mongolian, Syriac, etc.), this flag signifies
-    /// that it is safe to insert a U+0640 TATWEEL
-    /// character *before* this cluster for elongation.
-    pub const SAFE_TO_INSERT_TATWEEL: u32 = 0x00000004;
-
     /// All the currently defined flags.
-    pub const DEFINED: u32 = 0x00000007; // OR of all defined flags
+    pub const DEFINED: u32 = 0x00000003; // OR of all defined flags
 }
 
 /// Holds the positions of the glyph in both horizontal and vertical directions.
@@ -119,7 +113,7 @@ pub struct GlyphPosition {
     /// How much the glyph moves on the Y-axis before drawing it, this should
     /// not affect how much the line advances.
     pub y_offset: i32,
-    pub(crate) var: u32,
+    var: u32,
 }
 
 unsafe impl bytemuck::Zeroable for GlyphPosition {}
@@ -248,7 +242,7 @@ impl hb_glyph_info_t {
                     // https://github.com/harfbuzz/harfbuzz/issues/463
                     0xE0020..=0xE007F => props |= UnicodeProps::HIDDEN.bits(),
 
-                    // COMBINING GRAPHEME JOINER should not be skipped during GSUB either.
+                    // COMBINING GRAPHEME JOINER should not be skipped; at least some times.
                     // https://github.com/harfbuzz/harfbuzz/issues/554
                     0x034F => {
                         props |= UnicodeProps::HIDDEN.bits();
@@ -268,6 +262,11 @@ impl hb_glyph_info_t {
         self.set_unicode_props(props);
     }
 
+    #[inline]
+    pub(crate) fn is_hidden(&self) -> bool {
+        self.unicode_props() & UnicodeProps::HIDDEN.bits() != 0
+    }
+
     #[inline]
     pub(crate) fn unhide(&mut self) {
         let mut n = self.unicode_props();
@@ -336,16 +335,12 @@ pub struct hb_buffer_t {
     pub flags: BufferFlags,
     pub cluster_level: hb_buffer_cluster_level_t,
     pub invisible: Option<GlyphId>,
-    pub not_found_variation_selector: Option<u32>,
 
     // Buffer contents.
     pub direction: Direction,
     pub script: Option<Script>,
     pub language: Option<Language>,
 
-    /// Shaping failure
-    pub shaping_failed: bool,
-
     /// Allocations successful.
     pub successful: bool,
     /// Whether we have an output buffer going on.
@@ -394,13 +389,11 @@ impl hb_buffer_t {
             cluster_level: HB_BUFFER_CLUSTER_LEVEL_DEFAULT,
             invisible: None,
             scratch_flags: HB_BUFFER_SCRATCH_FLAG_DEFAULT,
-            not_found_variation_selector: None,
             max_len: Self::MAX_LEN_DEFAULT,
             max_ops: Self::MAX_OPS_DEFAULT,
             direction: Direction::Invalid,
             script: None,
             language: None,
-            shaping_failed: false,
             successful: true,
             have_output: false,
             have_positions: false,
@@ -419,6 +412,11 @@ impl hb_buffer_t {
         }
     }
 
+    #[inline]
+    pub fn info_slice(&self) -> &[hb_glyph_info_t] {
+        &self.info[..self.len]
+    }
+
     #[inline]
     pub fn info_slice_mut(&mut self) -> &mut [hb_glyph_info_t] {
         &mut self.info[..self.len]
@@ -476,12 +474,6 @@ impl hb_buffer_t {
         &mut self.out_info_mut()[idx]
     }
 
-    pub fn digest(&self) -> hb_set_digest_t {
-        let mut digest = hb_set_digest_t::new();
-        digest.add_array(self.info.iter().map(|i| GlyphId(i.glyph_id as u16)));
-        digest
-    }
-
     fn clear(&mut self) {
         self.direction = Direction::Invalid;
         self.script = None;
@@ -507,7 +499,6 @@ impl hb_buffer_t {
         self.serial = 0;
         self.scratch_flags = HB_BUFFER_SCRATCH_FLAG_DEFAULT;
         self.cluster_level = HB_BUFFER_CLUSTER_LEVEL_DEFAULT;
-        self.not_found_variation_selector = None;
     }
 
     #[inline]
@@ -526,8 +517,7 @@ impl hb_buffer_t {
 
     #[inline]
     fn next_serial(&mut self) -> u8 {
-        // A `serial` overflow/wrap-around here is perfectly fine.
-        self.serial = self.serial.wrapping_add(1);
+        self.serial += 1;
 
         if self.serial == 0 {
             self.serial += 1;
@@ -537,9 +527,7 @@ impl hb_buffer_t {
     }
 
     fn add(&mut self, codepoint: u32, cluster: u32) {
-        if !self.ensure(self.len + 1) {
-            return;
-        }
+        self.ensure(self.len + 1);
 
         let i = self.len;
         self.info[i] = hb_glyph_info_t {
@@ -582,9 +570,8 @@ impl hb_buffer_t {
         }
 
         let mut start = 0;
-        let mut i = 1;
 
-        while i < self.len {
+        for i in 1..self.len {
             if !group(&self.info[i - 1], &self.info[i]) {
                 if merge_clusters {
                     self.merge_clusters(start, i);
@@ -594,16 +581,14 @@ impl hb_buffer_t {
                 start = i;
             }
 
-            i += 1;
+            if merge_clusters {
+                self.merge_clusters(start, i);
+            }
+
+            self.reverse_range(start, i);
+
+            self.reverse();
         }
-
-        if merge_clusters {
-            self.merge_clusters(start, i);
-        }
-
-        self.reverse_range(start, i);
-
-        self.reverse();
     }
 
     pub fn group_end<F>(&self, mut start: usize, group: F) -> usize
@@ -652,15 +637,15 @@ impl hb_buffer_t {
         // TODO: language must be set
     }
 
-    pub fn sync(&mut self) -> bool {
+    pub fn sync(&mut self) {
         assert!(self.have_output);
-        assert!(self.idx <= self.len);
 
+        assert!(self.idx <= self.len);
         if !self.successful {
             self.have_output = false;
             self.out_len = 0;
             self.idx = 0;
-            return false;
+            return;
         }
 
         self.next_glyphs(self.len - self.idx);
@@ -671,7 +656,6 @@ impl hb_buffer_t {
             let pos: Vec<hb_glyph_info_t> = bytemuck::cast_vec(core::mem::take(&mut self.pos));
             self.pos = info;
             self.info = pos;
-            self.have_separate_output = false;
         }
 
         self.len = self.out_len;
@@ -679,7 +663,6 @@ impl hb_buffer_t {
         self.have_output = false;
         self.out_len = 0;
         self.idx = 0;
-        true
     }
 
     pub fn clear_output(&mut self) {
@@ -838,14 +821,14 @@ impl hb_buffer_t {
         cluster_start: u32,
         cluster_end: u32,
     ) {
+        let not_mask = !mask;
+        value &= mask;
+
         if mask == 0 {
             return;
         }
 
-        let not_mask = !mask;
-        value &= mask;
-
-        if cluster_start == 0 && cluster_end == u32::MAX {
+        if cluster_start == 0 && cluster_end == core::u32::MAX {
             for info in &mut self.info[..self.len] {
                 info.mask = (info.mask & not_mask) | value;
             }
@@ -881,21 +864,17 @@ impl hb_buffer_t {
         }
 
         // Extend end
-        if cluster != self.info[end - 1].cluster {
-            while end < self.len && self.info[end - 1].cluster == self.info[end].cluster {
-                end += 1;
-            }
+        while end < self.len && self.info[end - 1].cluster == self.info[end].cluster {
+            end += 1;
         }
 
         // Extend start
-        if cluster != self.info[start].cluster {
-            while end < start && self.info[start - 1].cluster == self.info[start].cluster {
-                start -= 1;
-            }
+        while end < start && self.info[start - 1].cluster == self.info[start].cluster {
+            start -= 1;
         }
 
         // If we hit the start of buffer, continue in out-buffer.
-        if self.idx == start && self.info[start].cluster != cluster {
+        if self.idx == start {
             let mut i = self.out_len;
             while i != 0 && self.out_info()[i - 1].cluster == self.info[start].cluster {
                 Self::set_cluster(&mut self.out_info_mut()[i - 1], cluster, 0);
@@ -952,9 +931,7 @@ impl hb_buffer_t {
     pub fn delete_glyph(&mut self) {
         let cluster = self.info[self.idx].cluster;
 
-        if (self.idx + 1 < self.len && cluster == self.info[self.idx + 1].cluster)
-            || (self.out_len != 0 && cluster == self.out_info()[self.out_len - 1].cluster)
-        {
+        if self.idx + 1 < self.len && cluster == self.info[self.idx + 1].cluster {
             // Cluster survives; do nothing.
             self.skip_glyph();
             return;
@@ -1045,18 +1022,6 @@ impl hb_buffer_t {
         );
     }
 
-    pub fn safe_to_insert_tatweel(&mut self, start: Option<usize>, end: Option<usize>) {
-        if !self
-            .flags
-            .contains(BufferFlags::PRODUCE_SAFE_TO_INSERT_TATWEEL)
-        {
-            self.unsafe_to_break(start, end);
-            return;
-        }
-
-        self._set_glyph_flags(SAFE_TO_INSERT_TATWEEL, start, end, Some(true), None);
-    }
-
     /// Adds glyph flags in mask to infos with clusters between start and end.
     /// The start index will be from out-buffer if from_out_buffer is true.
     /// If interior is true, then the cluster having the minimum value is skipped. */
@@ -1085,8 +1050,10 @@ impl hb_buffer_t {
                     self.info[i].mask |= mask;
                 }
             } else {
-                let cluster = self._infos_find_min_cluster(&self.info, start, end, None);
-                self._infos_set_glyph_flags(false, start, end, cluster, mask);
+                let cluster = Self::_infos_find_min_cluster(&self.info, start, end, None);
+                if Self::_infos_set_glyph_flags(&mut self.info, start, end, cluster, mask) {
+                    self.scratch_flags |= HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS;
+                }
             }
         } else {
             assert!(start <= self.out_len);
@@ -1101,17 +1068,28 @@ impl hb_buffer_t {
                     self.info[i].mask |= mask;
                 }
             } else {
-                let mut cluster = self._infos_find_min_cluster(&self.info, self.idx, end, None);
-                cluster = self._infos_find_min_cluster(
-                    self.out_info(),
+                let mut cluster = Self::_infos_find_min_cluster(&self.info, self.idx, end, None);
+                cluster = Self::_infos_find_min_cluster(
+                    &self.out_info(),
                     start,
                     self.out_len,
                     Some(cluster),
                 );
 
                 let out_len = self.out_len;
-                self._infos_set_glyph_flags(true, start, out_len, cluster, mask);
-                self._infos_set_glyph_flags(false, self.idx, end, cluster, mask);
+                let first = Self::_infos_set_glyph_flags(
+                    &mut self.out_info_mut(),
+                    start,
+                    out_len,
+                    cluster,
+                    mask,
+                );
+                let second =
+                    Self::_infos_set_glyph_flags(&mut self.info, self.idx, end, cluster, mask);
+
+                if first || second {
+                    self.scratch_flags |= HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS;
+                }
             }
         }
     }
@@ -1121,10 +1099,14 @@ impl hb_buffer_t {
             return;
         }
 
-        self._set_glyph_flags(UNSAFE_TO_CONCAT, start, end, Some(false), None);
+        self._set_glyph_flags(UNSAFE_TO_CONCAT, start, end, Some(true), None);
     }
 
     pub fn unsafe_to_break_from_outbuffer(&mut self, start: Option<usize>, end: Option<usize>) {
+        if !self.flags.contains(BufferFlags::PRODUCE_UNSAFE_TO_CONCAT) {
+            return;
+        }
+
         self._set_glyph_flags(
             UNSAFE_TO_BREAK | UNSAFE_TO_CONCAT,
             start,
@@ -1135,10 +1117,6 @@ impl hb_buffer_t {
     }
 
     pub fn unsafe_to_concat_from_outbuffer(&mut self, start: Option<usize>, end: Option<usize>) {
-        if !self.flags.contains(BufferFlags::PRODUCE_UNSAFE_TO_CONCAT) {
-            return;
-        }
-
         self._set_glyph_flags(UNSAFE_TO_CONCAT, start, end, Some(false), Some(true));
     }
 
@@ -1195,7 +1173,6 @@ impl hb_buffer_t {
         true
     }
 
-    #[must_use]
     pub fn ensure(&mut self, size: usize) -> bool {
         if size < self.len {
             return true;
@@ -1211,7 +1188,11 @@ impl hb_buffer_t {
         true
     }
 
-    #[must_use]
+    pub fn set_len(&mut self, len: usize) {
+        self.ensure(len);
+        self.len = len;
+    }
+
     fn make_room_for(&mut self, num_in: usize, num_out: usize) -> bool {
         if !self.ensure(self.out_len + num_out) {
             return false;
@@ -1231,9 +1212,7 @@ impl hb_buffer_t {
 
     fn shift_forward(&mut self, count: usize) {
         assert!(self.have_output);
-        if !self.ensure(self.len + count) {
-            return;
-        }
+        self.ensure(self.len + count);
 
         for i in (0..(self.len - self.idx)).rev() {
             self.info[self.idx + count + i] = self.info[self.idx + i];
@@ -1296,7 +1275,6 @@ impl hb_buffer_t {
     // Called around shape()
     pub(crate) fn enter(&mut self) {
         self.serial = 0;
-        self.shaping_failed = false;
         self.scratch_flags = HB_BUFFER_SCRATCH_FLAG_DEFAULT;
 
         if let Some(len) = self.len.checked_mul(hb_buffer_t::MAX_LEN_FACTOR) {
@@ -1315,99 +1293,44 @@ impl hb_buffer_t {
         self.max_len = hb_buffer_t::MAX_LEN_DEFAULT;
         self.max_ops = hb_buffer_t::MAX_OPS_DEFAULT;
         self.serial = 0;
-        // Intentionally not resetting shaping_failed, such that it can be inspected.
     }
 
     fn _infos_find_min_cluster(
-        &self,
         info: &[hb_glyph_info_t],
         start: usize,
         end: usize,
         cluster: Option<u32>,
     ) -> u32 {
-        let mut cluster = cluster.unwrap_or(u32::MAX);
+        let mut cluster = cluster.unwrap_or(core::u32::MAX);
 
-        if start == end {
-            return cluster;
+        for glyph_info in &info[start..end] {
+            cluster = core::cmp::min(cluster, glyph_info.cluster);
         }
 
-        if self.cluster_level == HB_BUFFER_CLUSTER_LEVEL_MONOTONE_CHARACTERS {
-            for glyph_info in &info[start..end] {
-                cluster = core::cmp::min(cluster, glyph_info.cluster);
-            }
-        }
-
-        cluster.min(info[start].cluster.min(info[end - 1].cluster))
+        cluster
     }
 
+    #[must_use]
     fn _infos_set_glyph_flags(
-        &mut self,
-        out_info: bool,
+        info: &mut [hb_glyph_info_t],
         start: usize,
         end: usize,
         cluster: u32,
         mask: hb_mask_t,
-    ) {
-        let mut apply_scratch_flags = false;
-
-        if start == end {
-            return;
-        }
-
-        let cluster_level = self.cluster_level;
-
-        let infos = if out_info {
-            self.out_info_mut()
-        } else {
-            self.info.as_mut_slice()
-        };
-
-        let cluster_first = infos[start].cluster;
-        let cluster_last = infos[end - 1].cluster;
-
-        if cluster_level == HB_BUFFER_CLUSTER_LEVEL_CHARACTERS
-            || (cluster != cluster_first && cluster != cluster_last)
-        {
-            for i in start..end {
-                if infos[i].cluster != cluster {
-                    apply_scratch_flags = true;
-                    infos[i].mask |= mask;
-                }
-            }
-
-            if apply_scratch_flags {
-                self.scratch_flags |= HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS;
-            }
-
-            return;
-        }
-
-        // Monotone clusters
-        if cluster == cluster_first {
-            let mut i = end;
-            while start < i && infos[i - 1].cluster != cluster_first {
-                if cluster != infos[i - 1].cluster {
-                    apply_scratch_flags = true;
-                    infos[i - 1].mask |= mask;
-                }
-
-                i -= 1;
-            }
-        } else {
-            let mut i = start;
-            while i < end && infos[i].cluster != cluster_last {
-                if cluster != infos[i].cluster {
-                    apply_scratch_flags = true;
-                    infos[i].mask |= mask;
-                }
-
-                i += 1;
+    ) -> bool {
+        // NOTE: Because of problems with ownership, we don't pass the scratch flags to this
+        // function, unlike in harfbuzz. Because of this, each time you call this function you
+        // the caller needs to set the "BufferScratchFlags::HAS_GLYPH_FLAGS" scratch flag
+        // themselves if the function returns true.
+        let mut unsafe_to_break = false;
+        for glyph_info in &mut info[start..end] {
+            if glyph_info.cluster != cluster {
+                glyph_info.mask |= mask;
+                unsafe_to_break = true;
             }
         }
 
-        if apply_scratch_flags {
-            self.scratch_flags |= HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS;
-        }
+        unsafe_to_break
     }
 
     /// Checks that buffer contains no elements.
@@ -1416,9 +1339,7 @@ impl hb_buffer_t {
     }
 
     fn push_str(&mut self, text: &str) {
-        if !self.ensure(self.len + text.chars().count()) {
-            return;
-        }
+        self.ensure(self.len + text.chars().count());
 
         for (i, c) in text.char_indices() {
             self.add(c as u32, i as u32);
@@ -1516,14 +1437,13 @@ bitflags::bitflags! {
     pub struct UnicodeProps: u16 {
         const GENERAL_CATEGORY  = 0x001F;
         const IGNORABLE         = 0x0020;
-        // MONGOLIAN FREE VARIATION SELECTOR 1..4, or TAG characters, or CGJ sometimes
+        // MONGOLIAN FREE VARIATION SELECTOR 1..4, or TAG characters
         const HIDDEN            = 0x0040;
         const CONTINUATION      = 0x0080;
 
         // If GEN_CAT=FORMAT, top byte masks:
         const CF_ZWJ            = 0x0100;
         const CF_ZWNJ           = 0x0200;
-        const CF_VS           = 0x0400;
     }
 }
 
@@ -1553,14 +1473,12 @@ pub const HB_BUFFER_SCRATCH_FLAG_HAS_SPACE_FALLBACK: u32 = 0x00000004;
 pub const HB_BUFFER_SCRATCH_FLAG_HAS_GPOS_ATTACHMENT: u32 = 0x00000008;
 pub const HB_BUFFER_SCRATCH_FLAG_HAS_CGJ: u32 = 0x00000010;
 pub const HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS: u32 = 0x00000020;
-pub const HB_BUFFER_SCRATCH_FLAG_HAS_BROKEN_SYLLABLE: u32 = 0x00000040;
-pub const HB_BUFFER_SCRATCH_FLAG_HAS_VARIATION_SELECTOR_FALLBACK: u32 = 0x00000080;
 
-/* Reserved for shapers' internal use. */
-pub const HB_BUFFER_SCRATCH_FLAG_SHAPER0: u32 = 0x01000000;
-// pub const HB_BUFFER_SCRATCH_FLAG_SHAPER1: u32 = 0x02000000;
-// pub const HB_BUFFER_SCRATCH_FLAG_SHAPER2: u32 = 0x04000000;
-// pub const HB_BUFFER_SCRATCH_FLAG_SHAPER3: u32 = 0x08000000;
+/* Reserved for complex shapers' internal use. */
+pub const HB_BUFFER_SCRATCH_FLAG_COMPLEX0: u32 = 0x01000000;
+// pub const HB_BUFFER_SCRATCH_FLAG_COMPLEX1: u32 = 0x02000000;
+// pub const HB_BUFFER_SCRATCH_FLAG_COMPLEX2: u32 = 0x04000000;
+// pub const HB_BUFFER_SCRATCH_FLAG_COMPLEX3: u32 = 0x08000000;
 
 /// A buffer that contains an input string ready for shaping.
 pub struct UnicodeBuffer(pub(crate) hb_buffer_t);
@@ -1641,12 +1559,6 @@ impl UnicodeBuffer {
         self.0.language = Some(lang);
     }
 
-    /// Set the glyph value to replace not-found variation-selector characters with.
-    #[inline]
-    pub fn set_not_found_variation_selector_glyph(&mut self, glyph: u32) {
-        self.0.not_found_variation_selector = Some(glyph)
-    }
-
     /// Get the buffer language.
     #[inline]
     pub fn language(&self) -> Option<Language> {
@@ -1815,7 +1727,7 @@ impl GlyphBuffer {
             }
 
             if flags.contains(SerializeFlags::GLYPH_EXTENTS) {
-                let mut extents = hb_glyph_extents_t::default();
+                let mut extents = GlyphExtents::default();
                 face.glyph_extents(info.as_glyph(), &mut extents);
                 write!(
                     &mut s,
