@@ -1,14 +1,20 @@
-use super::aat_map;
 use super::buffer::*;
 use super::ot_layout::*;
 use super::ot_layout_gpos_table::GPOS;
 use super::ot_map::*;
-use super::ot_shape_complex::*;
 use super::ot_shape_plan::hb_ot_shape_plan_t;
+use super::ot_shaper::*;
 use super::unicode::{hb_unicode_general_category_t, CharExt, GeneralCategoryExt};
 use super::*;
 use super::{hb_font_t, hb_tag_t};
 use crate::hb::aat_layout::hb_aat_layout_remove_deleted_glyphs;
+use crate::hb::algs::{rb_flag, rb_flag_unsafe};
+use crate::hb::buffer::glyph_flag::{SAFE_TO_INSERT_TATWEEL, UNSAFE_TO_BREAK, UNSAFE_TO_CONCAT};
+use crate::hb::unicode::hb_gc::{
+    RB_UNICODE_GENERAL_CATEGORY_LOWERCASE_LETTER, RB_UNICODE_GENERAL_CATEGORY_OTHER_LETTER,
+    RB_UNICODE_GENERAL_CATEGORY_SPACE_SEPARATOR, RB_UNICODE_GENERAL_CATEGORY_TITLECASE_LETTER,
+    RB_UNICODE_GENERAL_CATEGORY_UPPERCASE_LETTER,
+};
 use crate::BufferFlags;
 use crate::{Direction, Feature, Language, Script};
 
@@ -17,11 +23,10 @@ pub struct hb_ot_shape_planner_t<'a> {
     pub direction: Direction,
     pub script: Option<Script>,
     pub ot_map: hb_ot_map_builder_t<'a>,
-    pub aat_map: aat_map::hb_aat_map_builder_t,
     pub apply_morx: bool,
     pub script_zero_marks: bool,
     pub script_fallback_mark_positioning: bool,
-    pub shaper: &'static hb_ot_complex_shaper_t,
+    pub shaper: &'static hb_ot_shaper_t,
 }
 
 impl<'a> hb_ot_shape_planner_t<'a> {
@@ -32,7 +37,6 @@ impl<'a> hb_ot_shape_planner_t<'a> {
         language: Option<&Language>,
     ) -> Self {
         let ot_map = hb_ot_map_builder_t::new(face, script, language);
-        let aat_map = aat_map::hb_aat_map_builder_t::default();
 
         let mut shaper = match script {
             Some(script) => hb_ot_shape_complex_categorize(
@@ -60,7 +64,6 @@ impl<'a> hb_ot_shape_planner_t<'a> {
             direction,
             script,
             ot_map,
-            aat_map,
             apply_morx,
             script_zero_marks,
             script_fallback_mark_positioning,
@@ -91,6 +94,8 @@ impl<'a> hb_ot_shape_planner_t<'a> {
 
         let empty = F_NONE;
 
+        self.ot_map.is_simple = true;
+
         self.ot_map
             .enable_feature(hb_tag_t::from_bytes(b"rvrn"), empty, 1);
         self.ot_map.add_gsub_pause(None);
@@ -138,6 +143,7 @@ impl<'a> hb_ot_shape_planner_t<'a> {
             .enable_feature(hb_tag_t::from_bytes(b"HARF"), empty, 1); // Considered discretionary.
 
         if let Some(func) = self.shaper.collect_features {
+            self.ot_map.is_simple = false;
             func(self);
         }
 
@@ -167,18 +173,15 @@ impl<'a> hb_ot_shape_planner_t<'a> {
                 .enable_feature(hb_tag_t::from_bytes(b"vert"), F_GLOBAL_SEARCH, 1);
         }
 
+        if !user_features.is_empty() {
+            self.ot_map.is_simple = false;
+        }
+
         for feature in user_features {
             let flags = if feature.is_global() { F_GLOBAL } else { empty };
             self.ot_map.add_feature(feature.tag, flags, feature.value);
         }
 
-        if self.apply_morx {
-            for feature in user_features {
-                self.aat_map
-                    .add_feature(self.face, feature.tag, feature.value);
-            }
-        }
-
         if let Some(func) = self.shaper.override_features {
             func(self);
         }
@@ -187,12 +190,6 @@ impl<'a> hb_ot_shape_planner_t<'a> {
     pub fn compile(mut self, user_features: &[Feature]) -> hb_ot_shape_plan_t {
         let ot_map = self.ot_map.compile();
 
-        let aat_map = if self.apply_morx {
-            self.aat_map.compile(self.face)
-        } else {
-            aat_map::hb_aat_map_t::default()
-        };
-
         let frac_mask = ot_map.get_1_mask(hb_tag_t::from_bytes(b"frac"));
         let numr_mask = ot_map.get_1_mask(hb_tag_t::from_bytes(b"numr"));
         let dnom_mask = ot_map.get_1_mask(hb_tag_t::from_bytes(b"dnom"));
@@ -230,7 +227,7 @@ impl<'a> hb_ot_shape_planner_t<'a> {
 
         // Decide who does positioning. GPOS, kerx, kern, or fallback.
         let has_kerx = self.face.tables().kerx.is_some();
-        let has_gsub = self.face.tables().gsub.is_some();
+        let has_gsub = !apply_morx && self.face.tables().gsub.is_some();
         let has_gpos = !disable_gpos && self.face.tables().gpos.is_some();
 
         // Prefer GPOS over kerx if GSUB is present;
@@ -278,7 +275,6 @@ impl<'a> hb_ot_shape_planner_t<'a> {
             script: self.script,
             shaper: self.shaper,
             ot_map,
-            aat_map,
             data: None,
             frac_mask,
             numr_mask,
@@ -311,7 +307,7 @@ impl<'a> hb_ot_shape_planner_t<'a> {
     }
 }
 
-pub struct ShapeContext<'a> {
+pub struct hb_ot_shape_context_t<'a> {
     pub plan: &'a hb_ot_shape_plan_t,
     pub face: &'a hb_font_t<'a>,
     pub buffer: &'a mut hb_buffer_t,
@@ -320,7 +316,7 @@ pub struct ShapeContext<'a> {
 }
 
 // Pull it all together!
-pub fn shape_internal(ctx: &mut ShapeContext) {
+pub fn shape_internal(ctx: &mut hb_ot_shape_context_t) {
     ctx.buffer.enter();
 
     initialize_masks(ctx);
@@ -345,20 +341,21 @@ pub fn shape_internal(ctx: &mut ShapeContext) {
     ctx.buffer.leave();
 }
 
-fn substitute_pre(ctx: &mut ShapeContext) {
+fn substitute_pre(ctx: &mut hb_ot_shape_context_t) {
     hb_ot_substitute_default(ctx);
-    hb_ot_substitute_complex(ctx);
+    hb_ot_substitute_plan(ctx);
 
-    if ctx.plan.apply_morx && !ctx.plan.apply_gpos {
-        hb_aat_layout_remove_deleted_glyphs(&mut ctx.buffer);
+    if ctx.plan.apply_morx && ctx.plan.apply_gpos {
+        hb_aat_layout_remove_deleted_glyphs(ctx.buffer);
     }
 }
 
-fn substitute_post(ctx: &mut ShapeContext) {
-    if ctx.plan.apply_morx && ctx.plan.apply_gpos {
+fn substitute_post(ctx: &mut hb_ot_shape_context_t) {
+    if ctx.plan.apply_morx && !ctx.plan.apply_gpos {
         aat_layout::hb_aat_layout_remove_deleted_glyphs(ctx.buffer);
     }
 
+    deal_with_variation_selectors(ctx.buffer);
     hide_default_ignorables(ctx.buffer, ctx.face);
 
     if let Some(func) = ctx.plan.shaper.postprocess_glyphs {
@@ -366,7 +363,7 @@ fn substitute_post(ctx: &mut ShapeContext) {
     }
 }
 
-fn hb_ot_substitute_default(ctx: &mut ShapeContext) {
+fn hb_ot_substitute_default(ctx: &mut hb_ot_shape_context_t) {
     rotate_chars(ctx);
 
     ot_shape_normalize::_hb_ot_shape_normalize(ctx.plan, ctx.buffer, ctx.face);
@@ -383,25 +380,21 @@ fn hb_ot_substitute_default(ctx: &mut ShapeContext) {
     map_glyphs_fast(ctx.buffer);
 }
 
-fn hb_ot_substitute_complex(ctx: &mut ShapeContext) {
+fn hb_ot_substitute_plan(ctx: &mut hb_ot_shape_context_t) {
     hb_ot_layout_substitute_start(ctx.face, ctx.buffer);
 
     if ctx.plan.fallback_glyph_classes {
         hb_synthesize_glyph_classes(ctx.buffer);
     }
 
-    substitute_by_plan(ctx.plan, ctx.face, ctx.buffer);
-}
-
-fn substitute_by_plan(plan: &hb_ot_shape_plan_t, face: &hb_font_t, buffer: &mut hb_buffer_t) {
-    if plan.apply_morx {
-        aat_layout::hb_aat_layout_substitute(plan, face, buffer);
+    if ctx.plan.apply_morx {
+        aat_layout::hb_aat_layout_substitute(ctx.plan, ctx.face, ctx.buffer);
     } else {
-        super::ot_layout_gsub_table::substitute(plan, face, buffer);
+        super::ot_layout_gsub_table::substitute(ctx.plan, ctx.face, ctx.buffer);
     }
 }
 
-fn position(ctx: &mut ShapeContext) {
+fn position(ctx: &mut hb_ot_shape_context_t) {
     ctx.buffer.clear_positions();
 
     position_default(ctx);
@@ -413,7 +406,7 @@ fn position(ctx: &mut ShapeContext) {
     }
 }
 
-fn position_default(ctx: &mut ShapeContext) {
+fn position_default(ctx: &mut hb_ot_shape_context_t) {
     let len = ctx.buffer.len;
 
     if ctx.buffer.direction.is_horizontal() {
@@ -440,7 +433,7 @@ fn position_default(ctx: &mut ShapeContext) {
     }
 }
 
-fn position_complex(ctx: &mut ShapeContext) {
+fn position_complex(ctx: &mut hb_ot_shape_context_t) {
     // If the font has no GPOS and direction is forward, then when
     // zeroing mark widths, we shift the mark with it, such that the
     // mark is positioned hanging over the previous glyph.  When
@@ -497,7 +490,7 @@ fn position_by_plan(plan: &hb_ot_shape_plan_t, face: &hb_font_t, buffer: &mut hb
         aat_layout::hb_aat_layout_position(plan, face, buffer);
     }
     if plan.apply_kern {
-        super::kerning::kern(plan, face, buffer);
+        super::kerning::hb_ot_layout_kern(plan, face, buffer);
     } else if plan.apply_fallback_kern {
         ot_shape_fallback::_hb_ot_shape_fallback_kern(plan, face, buffer);
     }
@@ -507,12 +500,12 @@ fn position_by_plan(plan: &hb_ot_shape_plan_t, face: &hb_font_t, buffer: &mut hb
     }
 }
 
-fn initialize_masks(ctx: &mut ShapeContext) {
+fn initialize_masks(ctx: &mut hb_ot_shape_context_t) {
     let global_mask = ctx.plan.ot_map.get_global_mask();
     ctx.buffer.reset_masks(global_mask);
 }
 
-fn setup_masks(ctx: &mut ShapeContext) {
+fn setup_masks(ctx: &mut hb_ot_shape_context_t) {
     setup_masks_fraction(ctx);
 
     if let Some(func) = ctx.plan.shaper.setup_masks {
@@ -528,7 +521,7 @@ fn setup_masks(ctx: &mut ShapeContext) {
     }
 }
 
-fn setup_masks_fraction(ctx: &mut ShapeContext) {
+fn setup_masks_fraction(ctx: &mut hb_ot_shape_context_t) {
     let buffer = &mut ctx.buffer;
     if buffer.scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_NON_ASCII == 0 || !ctx.plan.has_frac {
         return;
@@ -567,6 +560,19 @@ fn setup_masks_fraction(ctx: &mut ShapeContext) {
                 end += 1;
             }
 
+            if start == i || end == i + 1 {
+                if start == i {
+                    buffer.unsafe_to_concat(Some(start), Some(start + 1));
+                }
+
+                if end == i + 1 {
+                    buffer.unsafe_to_concat(Some(end - 1), Some(end));
+                }
+
+                i += 1;
+                continue;
+            }
+
             buffer.unsafe_to_break(Some(start), Some(end));
 
             for info in &mut buffer.info[start..i] {
@@ -606,10 +612,23 @@ fn set_unicode_props(buffer: &mut hb_buffer_t) {
         let info = &mut later[0];
         info.init_unicode_props(&mut buffer.scratch_flags);
 
+        let gen_cat = _hb_glyph_info_get_general_category(info);
+
+        if (rb_flag_unsafe(gen_cat.to_rb())
+            & (rb_flag(RB_UNICODE_GENERAL_CATEGORY_LOWERCASE_LETTER)
+                | rb_flag(RB_UNICODE_GENERAL_CATEGORY_UPPERCASE_LETTER)
+                | rb_flag(RB_UNICODE_GENERAL_CATEGORY_TITLECASE_LETTER)
+                | rb_flag(RB_UNICODE_GENERAL_CATEGORY_OTHER_LETTER)
+                | rb_flag(RB_UNICODE_GENERAL_CATEGORY_SPACE_SEPARATOR)))
+            != 0
+        {
+            i += 1;
+            continue;
+        }
+
         // Marks are already set as continuation by the above line.
         // Handle Emoji_Modifier and ZWJ-continuation.
-        if _hb_glyph_info_get_general_category(info)
-            == hb_unicode_general_category_t::ModifierSymbol
+        if gen_cat == hb_unicode_general_category_t::ModifierSymbol
             && matches!(info.glyph_id, 0x1F3FB..=0x1F3FF)
         {
             _hb_glyph_info_set_continuation(info);
@@ -629,18 +648,20 @@ fn set_unicode_props(buffer: &mut hb_buffer_t) {
                     i += 1;
                 }
             }
-        } else if matches!(info.glyph_id, 0xE0020..=0xE007F) {
+        } else if matches!(info.glyph_id, 0xFF9E..=0xFF9F | 0xE0020..=0xE007F) {
             // Or part of the Other_Grapheme_Extend that is not marks.
-            // As of Unicode 11 that is just:
+            // As of Unicode 15 that is just:
             //
             // 200C          ; Other_Grapheme_Extend # Cf       ZERO WIDTH NON-JOINER
             // FF9E..FF9F    ; Other_Grapheme_Extend # Lm   [2] HALFWIDTH KATAKANA VOICED SOUND MARK..HALFWIDTH KATAKANA
             // SEMI-VOICED SOUND MARK E0020..E007F  ; Other_Grapheme_Extend # Cf  [96] TAG SPACE..CANCEL TAG
             //
             // ZWNJ is special, we don't want to merge it as there's no need, and keeping
-            // it separate results in more granular clusters.  Ignore Katakana for now.
+            // it separate results in more granular clusters.
             // Tags are used for Emoji sub-region flag sequences:
             // https://github.com/harfbuzz/harfbuzz/issues/1556
+            // Katakana ones were requested:
+            // https://github.com/harfbuzz/harfbuzz/issues/3844
             _hb_glyph_info_set_continuation(info);
         }
 
@@ -648,6 +669,18 @@ fn set_unicode_props(buffer: &mut hb_buffer_t) {
     }
 }
 
+pub(crate) fn syllabic_clear_var(
+    _: &hb_ot_shape_plan_t,
+    _: &hb_font_t,
+    buffer: &mut hb_buffer_t,
+) -> bool {
+    for info in &mut buffer.info {
+        info.set_syllable(0);
+    }
+
+    false
+}
+
 fn insert_dotted_circle(buffer: &mut hb_buffer_t, face: &hb_font_t) {
     if !buffer
         .flags
@@ -739,7 +772,7 @@ fn ensure_native_direction(buffer: &mut hb_buffer_t) {
     }
 }
 
-fn rotate_chars(ctx: &mut ShapeContext) {
+fn rotate_chars(ctx: &mut hb_ot_shape_context_t) {
     let len = ctx.buffer.len;
 
     if ctx.target_direction.is_backward() {
@@ -773,6 +806,10 @@ fn map_glyphs_fast(buffer: &mut hb_buffer_t) {
     for info in &mut buffer.info[..len] {
         info.glyph_id = info.glyph_index();
     }
+
+    for info in &mut buffer.out_info_mut()[..len] {
+        info.glyph_id = info.glyph_index();
+    }
 }
 
 fn hb_synthesize_glyph_classes(buffer: &mut hb_buffer_t) {
@@ -820,6 +857,33 @@ fn zero_width_default_ignorables(buffer: &mut hb_buffer_t) {
     }
 }
 
+fn deal_with_variation_selectors(buffer: &mut hb_buffer_t) {
+    if buffer.scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_VARIATION_SELECTOR_FALLBACK == 0 {
+        return;
+    }
+
+    // Note: In harfbuzz, this is part of the condition above (with OR), so it needs to stay
+    // in sync.
+    let Some(nf) = buffer.not_found_variation_selector else {
+        return;
+    };
+
+    let count = buffer.len;
+    let info = &mut buffer.info;
+    let pos = &mut buffer.pos;
+
+    for i in 0..count {
+        if _hb_glyph_info_is_variation_selector(&info[i]) {
+            info[i].glyph_id = nf;
+            pos[i].x_advance = 0;
+            pos[i].y_advance = 0;
+            pos[i].x_offset = 0;
+            pos[i].y_offset = 0;
+            _hb_glyph_info_set_variation_selector(&mut info[i], false);
+        }
+    }
+}
+
 fn zero_mark_widths_by_gdef(buffer: &mut hb_buffer_t, adjust_offsets: bool) {
     let len = buffer.len;
     for (info, pos) in buffer.info[..len].iter().zip(&mut buffer.pos[..len]) {
@@ -866,18 +930,47 @@ fn hide_default_ignorables(buffer: &mut hb_buffer_t, face: &hb_font_t) {
 fn propagate_flags(buffer: &mut hb_buffer_t) {
     // Propagate cluster-level glyph flags to be the same on all cluster glyphs.
     // Simplifies using them.
-    if buffer.scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS != 0 {
-        foreach_cluster!(buffer, start, end, {
-            let mut mask = 0;
-            for info in &buffer.info[start..end] {
-                mask |= info.mask * glyph_flag::DEFINED;
+
+    if buffer.scratch_flags & HB_BUFFER_SCRATCH_FLAG_HAS_GLYPH_FLAGS == 0 {
+        return;
+    }
+
+    /* If we are producing SAFE_TO_INSERT_TATWEEL, then do two things:
+     *
+     * - If the places that the Arabic shaper marked as SAFE_TO_INSERT_TATWEEL,
+     *   are UNSAFE_TO_BREAK, then clear the SAFE_TO_INSERT_TATWEEL,
+     * - Any place that is SAFE_TO_INSERT_TATWEEL, is also now UNSAFE_TO_BREAK.
+     *
+     * We couldn't make this interaction earlier. It has to be done here.
+     */
+    let flip_tatweel = buffer
+        .flags
+        .contains(BufferFlags::PRODUCE_SAFE_TO_INSERT_TATWEEL);
+
+    let clear_concat = !buffer.flags.contains(BufferFlags::PRODUCE_UNSAFE_TO_CONCAT);
+
+    foreach_cluster!(buffer, start, end, {
+        let mut mask = 0;
+        for info in &buffer.info[start..end] {
+            mask |= info.mask & glyph_flag::DEFINED;
+        }
+
+        if flip_tatweel {
+            if mask & UNSAFE_TO_BREAK != 0 {
+                mask &= !SAFE_TO_INSERT_TATWEEL;
             }
 
-            if mask != 0 {
-                for info in &mut buffer.info[start..end] {
-                    info.mask |= mask;
-                }
+            if mask & SAFE_TO_INSERT_TATWEEL != 0 {
+                mask |= UNSAFE_TO_BREAK | UNSAFE_TO_CONCAT;
             }
-        });
-    }
+        }
+
+        if clear_concat {
+            mask &= !UNSAFE_TO_CONCAT;
+
+            for info in &mut buffer.info[start..end] {
+                info.mask = mask;
+            }
+        }
+    });
 }
