@@ -50,7 +50,7 @@ fn select_4() {
                     .columns([Glyph::Image, Glyph::Aspect])
                     .from(Glyph::Table)
                     .take(),
-                Alias::new("subglyph")
+                "subglyph"
             )
             .to_string(SqliteQueryBuilder),
         r#"SELECT "image" FROM (SELECT "image", "aspect" FROM "glyph") AS "subglyph""#
@@ -443,7 +443,7 @@ fn select_31() {
 fn select_32() {
     assert_eq!(
         Query::select()
-            .expr_as(Expr::col(Char::Character), Alias::new("C"))
+            .expr_as(Expr::col(Char::Character), "C")
             .from(Char::Table)
             .to_string(SqliteQueryBuilder),
         r#"SELECT "character" AS "C" FROM "character""#
@@ -964,7 +964,7 @@ fn select_57() {
                 .case(Expr::col((Glyph::Table, Glyph::Aspect)).gt(0), "positive")
                 .case(Expr::col((Glyph::Table, Glyph::Aspect)).lt(0), "negative")
                 .finally("zero"),
-            Alias::new("polarity"),
+            "polarity",
         )
         .from(Glyph::Table)
         .to_owned();
@@ -1098,7 +1098,7 @@ fn insert_4() {
                 .unwrap()
                 .into()])
             .to_string(SqliteQueryBuilder),
-        r#"INSERT INTO "glyph" ("image") VALUES ('1970-01-01 00:00:00')"#
+        r#"INSERT INTO "glyph" ("image") VALUES ('1970-01-01 00:00:00.000000')"#
     );
 }
 
@@ -1800,19 +1800,19 @@ fn recursive_with_multiple_ctes() {
         .column(Asterisk)
         .from(Char::Table)
         .to_owned();
-    let sub_select1_name = SeaRc::new(Alias::new("sub1"));
+    let sub_select1_name = SeaRc::new("sub1");
     let mut sub_select1_cte = CommonTableExpression::new();
     sub_select1_cte.table_name(sub_select1_name.clone());
-    sub_select1_cte.column(SeaRc::new(Alias::new("a")));
+    sub_select1_cte.column(SeaRc::new("a"));
     sub_select1_cte.query(sub_select1);
     let sub_select2 = Query::select()
         .column(Asterisk)
         .from(Char::Table)
         .to_owned();
-    let sub_select2_name = SeaRc::new(Alias::new("sub2"));
+    let sub_select2_name = SeaRc::new("sub2");
     let mut sub_select2_cte = CommonTableExpression::new();
     sub_select2_cte.table_name(sub_select2_name.clone());
-    sub_select2_cte.column(SeaRc::new(Alias::new("b")));
+    sub_select2_cte.column(SeaRc::new("b"));
     sub_select2_cte.query(sub_select2);
 
     let mut with = WithClause::new();
