@@ -21,7 +21,7 @@
 )))]
 // Not needed for 2018 edition and conflicts with `rust_2018_idioms`
 #![doc(test(no_crate_inject))]
-#![doc(html_root_url = "https://docs.rs/serde_with/3.13.0/")]
+#![doc(html_root_url = "https://docs.rs/serde_with/3.15.0/")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![no_std]
 
@@ -29,7 +29,6 @@
 //! [![Build Status](https://github.com/jonasbb/serde_with/actions/workflows/ci.yaml/badge.svg)](https://github.com/jonasbb/serde_with)
 //! [![codecov](https://codecov.io/gh/jonasbb/serde_with/branch/master/graph/badge.svg)](https://codecov.io/gh/jonasbb/serde_with)
 //! [![CII Best Practices](https://bestpractices.coreinfrastructure.org/projects/4322/badge)](https://bestpractices.coreinfrastructure.org/projects/4322)
-//! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/py7ida)
 //!
 //! ---
 //!
@@ -75,7 +74,6 @@
 //!
 //! ## `DisplayFromStr`
 //!
-//! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/py7ida)
 //! ```rust
 //! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
@@ -110,7 +108,6 @@
 //!
 //! On top of it, `[u8; N]` (aka, bytes) can use the specialized `"Bytes"` for efficiency much like the `serde_bytes` crate.
 //!
-//! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/um0xyi)
 //! ```rust
 //! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
@@ -149,7 +146,6 @@
 //! If many fields are optional, putting the annotations on the structs can become tedious.
 //! The `#[skip_serializing_none]` attribute must be placed *before* the `#[derive]`.
 //!
-//! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/xr1tm0)
 //! ```rust
 //! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
@@ -257,24 +253,22 @@
 //! # }
 //! ```
 //!
-//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.13.0/serde_with/struct.DisplayFromStr.html
-//! [`with_prefix!`]: https://docs.rs/serde_with/3.13.0/serde_with/macro.with_prefix.html
-//! [`with_suffix!`]: https://docs.rs/serde_with/3.13.0/serde_with/macro.with_suffix.html
-//! [feature flags]: https://docs.rs/serde_with/3.13.0/serde_with/guide/feature_flags/index.html
-//! [skip_serializing_none]: https://docs.rs/serde_with/3.13.0/serde_with/attr.skip_serializing_none.html
-//! [StringWithSeparator]: https://docs.rs/serde_with/3.13.0/serde_with/struct.StringWithSeparator.html
-//! [user guide]: https://docs.rs/serde_with/3.13.0/serde_with/guide/index.html
+//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.15.0/serde_with/struct.DisplayFromStr.html
+//! [`with_prefix!`]: https://docs.rs/serde_with/3.15.0/serde_with/macro.with_prefix.html
+//! [`with_suffix!`]: https://docs.rs/serde_with/3.15.0/serde_with/macro.with_suffix.html
+//! [feature flags]: https://docs.rs/serde_with/3.15.0/serde_with/guide/feature_flags/index.html
+//! [skip_serializing_none]: https://docs.rs/serde_with/3.15.0/serde_with/attr.skip_serializing_none.html
+//! [StringWithSeparator]: https://docs.rs/serde_with/3.15.0/serde_with/struct.StringWithSeparator.html
+//! [user guide]: https://docs.rs/serde_with/3.15.0/serde_with/guide/index.html
 //! [with-annotation]: https://serde.rs/field-attrs.html#with
-//! [as-annotation]: https://docs.rs/serde_with/3.13.0/serde_with/guide/serde_as/index.html
+//! [as-annotation]: https://docs.rs/serde_with/3.15.0/serde_with/guide/serde_as/index.html
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
 #[doc(hidden)]
-pub extern crate core;
+extern crate core;
 #[doc(hidden)]
-pub extern crate serde;
-#[doc(hidden)]
-pub extern crate serde_derive;
+extern crate serde_core;
 #[cfg(feature = "std")]
 extern crate std;
 
@@ -299,7 +293,9 @@ mod duplicate_key_impls;
 #[cfg(feature = "alloc")]
 mod enum_map;
 #[cfg(feature = "std")]
-mod flatten_maybe;
+/// NOT PUBLIC API
+#[doc(hidden)]
+pub mod flatten_maybe;
 pub mod formats;
 #[cfg(feature = "hex")]
 #[cfg_attr(docsrs, doc(cfg(feature = "hex")))]
@@ -316,6 +312,9 @@ pub mod schemars_0_8;
 #[cfg(feature = "schemars_0_9")]
 #[cfg_attr(docsrs, doc(cfg(feature = "schemars_0_9")))]
 pub mod schemars_0_9;
+#[cfg(feature = "schemars_1")]
+#[cfg_attr(docsrs, doc(cfg(feature = "schemars_1")))]
+pub mod schemars_1;
 pub mod ser;
 mod serde_conv;
 #[cfg(feature = "time_0_3")]
@@ -399,14 +398,14 @@ pub(crate) mod prelude {
         fmt::{self, Display},
         hash::{BuildHasher, Hash},
         marker::PhantomData,
-        ops::Bound,
+        ops::{Bound, Range, RangeFrom, RangeInclusive, RangeTo},
         option::Option,
         pin::Pin,
         result::Result,
-        str::FromStr,
+        str::{self, FromStr},
         time::Duration,
     };
-    pub use serde::{
+    pub use serde_core::{
         de::{
             Deserialize, DeserializeOwned, DeserializeSeed, Deserializer, EnumAccess,
             Error as DeError, Expected, IgnoredAny, IntoDeserializer, MapAccess, SeqAccess,
@@ -495,7 +494,7 @@ pub use serde_with_macros::*;
 /// # }
 /// ```
 ///
-/// [serde_as]: https://docs.rs/serde_with/3.13.0/serde_with/attr.serde_as.html
+/// [serde_as]: https://docs.rs/serde_with/3.15.0/serde_with/attr.serde_as.html
 pub struct As<T: ?Sized>(PhantomData<T>);
 
 /// Adapter to convert from `serde_as` to the serde traits.
@@ -970,7 +969,7 @@ pub struct BytesOrString;
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.13.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.0/serde_with/guide/feature_flags/index.html
 pub struct DurationSeconds<
     FORMAT: formats::Format = u64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1102,7 +1101,7 @@ pub struct DurationSeconds<
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.13.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.0/serde_with/guide/feature_flags/index.html
 pub struct DurationSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1304,7 +1303,7 @@ pub struct DurationNanoSecondsWithFrac<
 /// [`SystemTime`]: std::time::SystemTime
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
-/// [feature flag]: https://docs.rs/serde_with/3.13.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.0/serde_with/guide/feature_flags/index.html
 pub struct TimestampSeconds<
     FORMAT: formats::Format = i64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1446,7 +1445,7 @@ pub struct TimestampSeconds<
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
 /// [NaiveDateTime]: ::chrono_0_4::NaiveDateTime
-/// [feature flag]: https://docs.rs/serde_with/3.13.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.15.0/serde_with/guide/feature_flags/index.html
 pub struct TimestampSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -2181,13 +2180,38 @@ pub struct TryFromIntoRef<T>(PhantomData<T>);
 #[cfg(feature = "alloc")]
 pub struct BorrowCow;
 
+/// A trait to inspect skipped deserialization errors
+///
+/// The [`VecSkipError`] and [`MapSkipError`] adapters allow to skip values which fail to deserialize.
+/// This trait allows inspecting these errors, for example for logging purposes.
+///
+/// The trait has a single method [`inspect_error`][InspectError::inspect_error], which will be called for each deserialization error.
+/// The default implementation for `()` does nothing.
+///
+/// See the documentation of [`VecSkipError`] and [`MapSkipError`] for usage examples.
+#[cfg(feature = "alloc")]
+pub trait InspectError {
+    /// Inspect a deserialization error which was skipped.
+    fn inspect_error(error: impl serde_core::de::Error);
+}
+
+#[cfg(feature = "alloc")]
+impl InspectError for () {
+    fn inspect_error(_error: impl serde_core::de::Error) {}
+}
+
 /// Deserialize a sequence into `Vec<T>`, skipping elements which fail to deserialize.
 ///
 /// The serialization behavior is identical to `Vec<T>`. This is an alternative to `Vec<T>`
 /// which is resilient against unexpected data.
 ///
+/// You can be notified of skipped elements by providing a type that implements the [`InspectError`] trait.
+/// The second generic argument `I` defaults to `()`, which does nothing.
+///
 /// # Examples
 ///
+/// ## Basic Usage
+///
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
@@ -2215,8 +2239,51 @@ pub struct BorrowCow;
 /// assert_eq!(data, serde_json::from_str(source_json).unwrap());
 /// # }
 /// ```
+///
+/// ## Using [`InspectError`](`crate::InspectError`) to log skipped elements
+///
+/// ```rust
+/// # #[cfg(all(feature = "macros", feature = "alloc"))] {
+/// # use serde::{Serialize, Deserialize};
+/// # use serde_with::{serde_as, InspectError, VecSkipError};
+/// # use std::cell::RefCell;
+///
+/// struct ErrorInspector;
+///
+/// thread_local! {
+///     static ERRORS: RefCell<Vec<String>> = RefCell::new(Vec::new());
+/// }
+///
+/// impl InspectError for ErrorInspector {
+///     fn inspect_error(error: impl serde::de::Error) {
+///         ERRORS.with(|errors| errors.borrow_mut().push(error.to_string()));
+///     }
+/// }
+///
+/// #[serde_as]
+/// #[derive(Debug, PartialEq, Deserialize, Serialize)]
+/// struct S {
+///     tag: String,
+///     #[serde_as(as = "VecSkipError<_, ErrorInspector>")]
+///     values: Vec<u8>,
+/// }
+///
+/// let json = r#"{"tag":"type","values":[0, "str", 1, [10, 11], -2, {}, 300]}"#;
+/// let s: S = serde_json::from_str(json).unwrap();
+/// assert_eq!(s.values, vec![0, 1]);
+///
+/// let errors = ERRORS.with(|errors| errors.borrow().clone());
+/// eprintln!("Errors: {errors:#?}");
+/// assert_eq!(errors.len(), 5);
+/// assert!(errors[0].contains("invalid type: string \"str\", expected u8"));
+/// assert!(errors[1].contains("invalid type: sequence, expected u8"));
+/// assert!(errors[2].contains("invalid value: integer `-2`, expected u8"));
+/// assert!(errors[3].contains("invalid type: map, expected u8"));
+/// assert!(errors[4].contains("invalid value: integer `300`, expected u8"));
+/// # }
+/// ```
 #[cfg(feature = "alloc")]
-pub struct VecSkipError<T>(PhantomData<T>);
+pub struct VecSkipError<T, I = ()>(PhantomData<(T, I)>);
 
 /// Deserialize a map, skipping keys and values which fail to deserialize.
 ///
@@ -2224,6 +2291,9 @@ pub struct VecSkipError<T>(PhantomData<T>);
 /// a map. Sometimes a map has heterogeneous keys or values but we only care about some specific
 /// types, and it is desirable to skip entries on errors.
 ///
+/// You can be notified of skipped elements by providing a type that implements the [`InspectError`] trait.
+/// The third generic argument `I` defaults to `()`, which does nothing.
+///
 /// It is especially useful in conjunction to `#[serde(flatten)]` to capture a map mixed in with
 /// other entries which we don't want to exhaust in the type definition.
 ///
@@ -2236,6 +2306,8 @@ pub struct VecSkipError<T>(PhantomData<T>);
 ///
 /// # Examples
 ///
+/// ## Basic Usage
+///
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
@@ -2273,8 +2345,51 @@ pub struct VecSkipError<T>(PhantomData<T>);
 /// assert_eq!(data, serde_json::from_str(source_json).unwrap());
 /// # }
 /// ```
+///
+/// ## Using [`InspectError`](`crate::InspectError`) to log skipped elements
+///
+/// ```rust
+/// # #[cfg(all(feature = "macros", feature = "alloc"))] {
+/// # use serde::{Serialize, Deserialize};
+/// # use serde_with::{serde_as, InspectError, MapSkipError};
+/// # use std::collections::BTreeMap;
+/// # use std::cell::RefCell;
+///
+/// struct ErrorInspector;
+///
+/// thread_local! {
+///     static ERRORS: RefCell<Vec<String>> = RefCell::new(Vec::new());
+/// }
+///
+/// impl InspectError for ErrorInspector {
+///     fn inspect_error(error: impl serde::de::Error) {
+///         ERRORS.with(|errors| errors.borrow_mut().push(error.to_string()));
+///     }
+/// }
+///
+/// #[serde_as]
+/// #[derive(Debug, PartialEq, Deserialize, Serialize)]
+/// struct S {
+///     tag: String,
+///     #[serde_as(as = "MapSkipError<_, _, ErrorInspector>")]
+///     values: BTreeMap<String, u8>,
+/// }
+///
+/// let json = r#"{"tag":"type","values":{"valid":42,"invalid": null,"another":"str","nested":[1,2,3]}}"#;
+/// let s: S = serde_json::from_str(json).unwrap();
+/// assert_eq!(s.values.len(), 1);
+/// assert_eq!(s.values.get("valid"), Some(&42));
+///
+/// let errors = ERRORS.with(|errors| errors.borrow().clone());
+/// assert_eq!(errors.len(), 3);
+/// eprintln!("Errors: {errors:#?}");
+/// assert!(errors[0].contains("invalid type: null, expected u8"));
+/// assert!(errors[1].contains("invalid type: string \"str\", expected u8"));
+/// assert!(errors[2].contains("invalid type: sequence, expected u8"));
+/// # }
+/// ```
 #[cfg(feature = "alloc")]
-pub struct MapSkipError<K, V>(PhantomData<(K, V)>);
+pub struct MapSkipError<K, V, I = ()>(PhantomData<(K, V, I)>);
 
 /// Deserialize a boolean from a number
 ///
@@ -2598,6 +2713,10 @@ pub struct SetLastValueWins<T>(PhantomData<T>);
 /// It is added implicitly by the [`#[serde_as]`](crate::serde_as) macro when any `schemars`
 /// feature is enabled.
 ///
-/// [`JsonSchema`]: ::schemars_0_8::JsonSchema
-#[cfg(any(feature = "schemars_0_8", feature = "schemars_0_9"))]
+/// [`JsonSchema`]: ::schemars_1::JsonSchema
+#[cfg(any(
+    feature = "schemars_0_8",
+    feature = "schemars_0_9",
+    feature = "schemars_1"
+))]
 pub struct Schema<T: ?Sized, TA>(PhantomData<T>, PhantomData<TA>);
