@@ -15,21 +15,28 @@ pub use self::alignment::{AlignContent, AlignItems, AlignSelf, JustifyContent, J
 pub use self::available_space::AvailableSpace;
 pub use self::compact_length::CompactLength;
 pub use self::dimension::{Dimension, LengthPercentage, LengthPercentageAuto};
+use crate::sys::DefaultCheapStr;
 
 #[cfg(feature = "block_layout")]
 pub use self::block::{BlockContainerStyle, BlockItemStyle, TextAlign};
 #[cfg(feature = "flexbox")]
 pub use self::flex::{FlexDirection, FlexWrap, FlexboxContainerStyle, FlexboxItemStyle};
 #[cfg(feature = "grid")]
-pub(crate) use self::grid::{GenericGridPlacement, OriginZeroGridPlacement};
-#[cfg(feature = "grid")]
 pub use self::grid::{
-    GridAutoFlow, GridContainerStyle, GridItemStyle, GridPlacement, GridTrackRepetition, MaxTrackSizingFunction,
-    MinTrackSizingFunction, NonRepeatedTrackSizingFunction, TrackSizingFunction,
+    GenericGridPlacement, GenericGridTemplateComponent, GenericRepetition, GridAutoFlow, GridContainerStyle,
+    GridItemStyle, GridPlacement, GridTemplateComponent, GridTemplateRepetition, MaxTrackSizingFunction,
+    MinTrackSizingFunction, RepetitionCount, TrackSizingFunction,
 };
+#[cfg(feature = "grid")]
+pub(crate) use self::grid::{GridAreaAxis, GridAreaEnd};
+#[cfg(feature = "grid")]
+pub use self::grid::{GridTemplateArea, NamedGridLine, TemplateLineNames};
+#[cfg(feature = "grid")]
+pub(crate) use self::grid::{NonNamedGridPlacement, OriginZeroGridPlacement};
 
 use crate::geometry::{Point, Rect, Size};
 use crate::style_helpers::TaffyAuto as _;
+use core::fmt::Debug;
 
 #[cfg(feature = "grid")]
 use crate::geometry::Line;
@@ -38,12 +45,37 @@ use crate::style_helpers;
 #[cfg(feature = "grid")]
 use crate::util::sys::GridTrackVec;
 
+use crate::sys::String;
+
+/// Trait that represents a cheaply clonable string. If you're unsure what to use here
+/// consider `Arc<str>` or `string_cache::Atom`.
+#[cfg(any(feature = "alloc", feature = "std"))]
+pub trait CheapCloneStr:
+    AsRef<str> + for<'a> From<&'a str> + From<String> + PartialEq + Eq + Clone + Default + Debug + 'static
+{
+}
+#[cfg(any(feature = "alloc", feature = "std"))]
+impl<T> CheapCloneStr for T where
+    T: AsRef<str> + for<'a> From<&'a str> + From<String> + PartialEq + Eq + Clone + Default + Debug + 'static
+{
+}
+
+/// Trait that represents a cheaply clonable string. If you're unsure what to use here
+/// consider `Arc<str>` or `string_cache::Atom`.
+#[cfg(not(any(feature = "alloc", feature = "std")))]
+pub trait CheapCloneStr {}
+#[cfg(not(any(feature = "alloc", feature = "std")))]
+impl<T> CheapCloneStr for T {}
+
 /// The core set of styles that are shared between all CSS layout nodes
 ///
 /// Note that all methods come with a default implementation which simply returns the default value for that style property
 /// but this is a just a convenience to save on boilerplate for styles that your implementation doesn't support. You will need
 /// to override the default implementation for each style property that your style type actually supports.
 pub trait CoreStyle {
+    /// The type of custom identifiers used to identify named grid lines and areas
+    type CustomIdent: CheapCloneStr;
+
     /// Which box generation mode should be used
     #[inline(always)]
     fn box_generation_mode(&self) -> BoxGenerationMode {
@@ -70,7 +102,7 @@ pub trait CoreStyle {
     /// How children overflowing their container should affect layout
     #[inline(always)]
     fn overflow(&self) -> Point<Overflow> {
-        Style::DEFAULT.overflow
+        Style::<Self::CustomIdent>::DEFAULT.overflow
     }
     /// How much space (in points) should be reserved for the scrollbars of `Overflow::Scroll` and `Overflow::Auto` nodes.
     #[inline(always)]
@@ -82,52 +114,52 @@ pub trait CoreStyle {
     /// What should the `position` value of this struct use as a base offset?
     #[inline(always)]
     fn position(&self) -> Position {
-        Style::DEFAULT.position
+        Style::<Self::CustomIdent>::DEFAULT.position
     }
     /// How should the position of this element be tweaked relative to the layout defined?
     #[inline(always)]
     fn inset(&self) -> Rect<LengthPercentageAuto> {
-        Style::DEFAULT.inset
+        Style::<Self::CustomIdent>::DEFAULT.inset
     }
 
     // Size properies
     /// Sets the initial size of the item
     #[inline(always)]
     fn size(&self) -> Size<Dimension> {
-        Style::DEFAULT.size
+        Style::<Self::CustomIdent>::DEFAULT.size
     }
     /// Controls the minimum size of the item
     #[inline(always)]
     fn min_size(&self) -> Size<Dimension> {
-        Style::DEFAULT.min_size
+        Style::<Self::CustomIdent>::DEFAULT.min_size
     }
     /// Controls the maximum size of the item
     #[inline(always)]
     fn max_size(&self) -> Size<Dimension> {
-        Style::DEFAULT.max_size
+        Style::<Self::CustomIdent>::DEFAULT.max_size
     }
     /// Sets the preferred aspect ratio for the item
     /// The ratio is calculated as width divided by height.
     #[inline(always)]
     fn aspect_ratio(&self) -> Option<f32> {
-        Style::DEFAULT.aspect_ratio
+        Style::<Self::CustomIdent>::DEFAULT.aspect_ratio
     }
 
     // Spacing Properties
     /// How large should the margin be on each side?
     #[inline(always)]
     fn margin(&self) -> Rect<LengthPercentageAuto> {
-        Style::DEFAULT.margin
+        Style::<Self::CustomIdent>::DEFAULT.margin
     }
     /// How large should the padding be on each side?
     #[inline(always)]
     fn padding(&self) -> Rect<LengthPercentage> {
-        Style::DEFAULT.padding
+        Style::<Self::CustomIdent>::DEFAULT.padding
     }
     /// How large should the border be on each side?
     #[inline(always)]
     fn border(&self) -> Rect<LengthPercentage> {
-        Style::DEFAULT.border
+        Style::<Self::CustomIdent>::DEFAULT.border
     }
 }
 
@@ -338,7 +370,10 @@ impl Overflow {
 #[derive(Clone, PartialEq, Debug)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "serde", serde(default))]
-pub struct Style {
+pub struct Style<S: CheapCloneStr = DefaultCheapStr> {
+    /// This is a dummy field which is necessary to make Taffy compile with the `grid` feature disabled
+    /// It should always be set to `core::marker::PhantomData`.
+    pub dummy: core::marker::PhantomData<S>,
     /// What layout strategy should be used?
     pub display: Display,
     /// Whether a child is display:table or not. This affects children of block layouts.
@@ -446,32 +481,44 @@ pub struct Style {
     // Grid container properies
     /// Defines the track sizing functions (heights) of the grid rows
     #[cfg(feature = "grid")]
-    pub grid_template_rows: GridTrackVec<TrackSizingFunction>,
+    pub grid_template_rows: GridTrackVec<GridTemplateComponent<S>>,
     /// Defines the track sizing functions (widths) of the grid columns
     #[cfg(feature = "grid")]
-    pub grid_template_columns: GridTrackVec<TrackSizingFunction>,
+    pub grid_template_columns: GridTrackVec<GridTemplateComponent<S>>,
     /// Defines the size of implicitly created rows
     #[cfg(feature = "grid")]
-    pub grid_auto_rows: GridTrackVec<NonRepeatedTrackSizingFunction>,
+    pub grid_auto_rows: GridTrackVec<TrackSizingFunction>,
     /// Defined the size of implicitly created columns
     #[cfg(feature = "grid")]
-    pub grid_auto_columns: GridTrackVec<NonRepeatedTrackSizingFunction>,
+    pub grid_auto_columns: GridTrackVec<TrackSizingFunction>,
     /// Controls how items get placed into the grid for auto-placed items
     #[cfg(feature = "grid")]
     pub grid_auto_flow: GridAutoFlow,
 
+    // Grid container named properties
+    /// Defines the rectangular grid areas
+    #[cfg(feature = "grid")]
+    pub grid_template_areas: GridTrackVec<GridTemplateArea<S>>,
+    /// The named lines between the columns
+    #[cfg(feature = "grid")]
+    pub grid_template_column_names: GridTrackVec<GridTrackVec<S>>,
+    /// The named lines between the rows
+    #[cfg(feature = "grid")]
+    pub grid_template_row_names: GridTrackVec<GridTrackVec<S>>,
+
     // Grid child properties
     /// Defines which row in the grid the item should start and end at
     #[cfg(feature = "grid")]
-    pub grid_row: Line<GridPlacement>,
+    pub grid_row: Line<GridPlacement<S>>,
     /// Defines which column in the grid the item should start and end at
     #[cfg(feature = "grid")]
-    pub grid_column: Line<GridPlacement>,
+    pub grid_column: Line<GridPlacement<S>>,
 }
 
-impl Style {
+impl<S: CheapCloneStr> Style<S> {
     /// The [`Default`] layout, in a form that can be used in const functions
-    pub const DEFAULT: Style = Style {
+    pub const DEFAULT: Style<S> = Style {
+        dummy: core::marker::PhantomData,
         display: Display::DEFAULT,
         item_is_table: false,
         item_is_replaced: false,
@@ -522,25 +569,33 @@ impl Style {
         #[cfg(feature = "grid")]
         grid_template_columns: GridTrackVec::new(),
         #[cfg(feature = "grid")]
+        grid_template_areas: GridTrackVec::new(),
+        #[cfg(feature = "grid")]
+        grid_template_column_names: GridTrackVec::new(),
+        #[cfg(feature = "grid")]
+        grid_template_row_names: GridTrackVec::new(),
+        #[cfg(feature = "grid")]
         grid_auto_rows: GridTrackVec::new(),
         #[cfg(feature = "grid")]
         grid_auto_columns: GridTrackVec::new(),
         #[cfg(feature = "grid")]
         grid_auto_flow: GridAutoFlow::Row,
         #[cfg(feature = "grid")]
-        grid_row: Line { start: GridPlacement::Auto, end: GridPlacement::Auto },
+        grid_row: Line { start: GridPlacement::<S>::Auto, end: GridPlacement::<S>::Auto },
         #[cfg(feature = "grid")]
-        grid_column: Line { start: GridPlacement::Auto, end: GridPlacement::Auto },
+        grid_column: Line { start: GridPlacement::<S>::Auto, end: GridPlacement::<S>::Auto },
     };
 }
 
-impl Default for Style {
+impl<S: CheapCloneStr> Default for Style<S> {
     fn default() -> Self {
         Style::DEFAULT
     }
 }
 
-impl CoreStyle for Style {
+impl<S: CheapCloneStr> CoreStyle for Style<S> {
+    type CustomIdent = S;
+
     #[inline(always)]
     fn box_generation_mode(&self) -> BoxGenerationMode {
         match self.display {
@@ -608,6 +663,8 @@ impl CoreStyle for Style {
 }
 
 impl<T: CoreStyle> CoreStyle for &'_ T {
+    type CustomIdent = T::CustomIdent;
+
     #[inline(always)]
     fn box_generation_mode(&self) -> BoxGenerationMode {
         (*self).box_generation_mode()
@@ -671,7 +728,7 @@ impl<T: CoreStyle> CoreStyle for &'_ T {
 }
 
 #[cfg(feature = "block_layout")]
-impl BlockContainerStyle for Style {
+impl<S: CheapCloneStr> BlockContainerStyle for Style<S> {
     #[inline(always)]
     fn text_align(&self) -> TextAlign {
         self.text_align
@@ -687,7 +744,7 @@ impl<T: BlockContainerStyle> BlockContainerStyle for &'_ T {
 }
 
 #[cfg(feature = "block_layout")]
-impl BlockItemStyle for Style {
+impl<S: CheapCloneStr> BlockItemStyle for Style<S> {
     #[inline(always)]
     fn is_table(&self) -> bool {
         self.item_is_table
@@ -703,7 +760,7 @@ impl<T: BlockItemStyle> BlockItemStyle for &'_ T {
 }
 
 #[cfg(feature = "flexbox")]
-impl FlexboxContainerStyle for Style {
+impl<S: CheapCloneStr> FlexboxContainerStyle for Style<S> {
     #[inline(always)]
     fn flex_direction(&self) -> FlexDirection {
         self.flex_direction
@@ -759,7 +816,7 @@ impl<T: FlexboxContainerStyle> FlexboxContainerStyle for &'_ T {
 }
 
 #[cfg(feature = "flexbox")]
-impl FlexboxItemStyle for Style {
+impl<S: CheapCloneStr> FlexboxItemStyle for Style<S> {
     #[inline(always)]
     fn flex_basis(&self) -> Dimension {
         self.flex_basis
@@ -799,31 +856,51 @@ impl<T: FlexboxItemStyle> FlexboxItemStyle for &'_ T {
 }
 
 #[cfg(feature = "grid")]
-impl GridContainerStyle for Style {
-    type TemplateTrackList<'a>
-        = &'a [TrackSizingFunction]
+impl<S: CheapCloneStr> GridContainerStyle for Style<S> {
+    type Repetition<'a>
+        = &'a GridTemplateRepetition<S>
     where
         Self: 'a;
+
+    type TemplateTrackList<'a>
+        = core::iter::Map<
+        core::slice::Iter<'a, GridTemplateComponent<S>>,
+        fn(&'a GridTemplateComponent<S>) -> GenericGridTemplateComponent<S, &'a GridTemplateRepetition<S>>,
+    >
+    where
+        Self: 'a;
+
     type AutoTrackList<'a>
-        = &'a [NonRepeatedTrackSizingFunction]
+        = core::iter::Copied<core::slice::Iter<'a, TrackSizingFunction>>
+    where
+        Self: 'a;
+
+    #[cfg(feature = "grid")]
+    type TemplateLineNames<'a>
+        = core::iter::Map<core::slice::Iter<'a, GridTrackVec<S>>, fn(&GridTrackVec<S>) -> core::slice::Iter<'_, S>>
+    where
+        Self: 'a;
+    #[cfg(feature = "grid")]
+    type GridTemplateAreas<'a>
+        = core::iter::Cloned<core::slice::Iter<'a, GridTemplateArea<S>>>
     where
         Self: 'a;
 
     #[inline(always)]
-    fn grid_template_rows(&self) -> &[TrackSizingFunction] {
-        &self.grid_template_rows
+    fn grid_template_rows(&self) -> Option<Self::TemplateTrackList<'_>> {
+        Some(self.grid_template_rows.iter().map(|c| c.as_component_ref()))
     }
     #[inline(always)]
-    fn grid_template_columns(&self) -> &[TrackSizingFunction] {
-        &self.grid_template_columns
+    fn grid_template_columns(&self) -> Option<Self::TemplateTrackList<'_>> {
+        Some(self.grid_template_columns.iter().map(|c| c.as_component_ref()))
     }
     #[inline(always)]
-    fn grid_auto_rows(&self) -> &[NonRepeatedTrackSizingFunction] {
-        &self.grid_auto_rows
+    fn grid_auto_rows(&self) -> Self::AutoTrackList<'_> {
+        self.grid_auto_rows.iter().copied()
     }
     #[inline(always)]
-    fn grid_auto_columns(&self) -> &[NonRepeatedTrackSizingFunction] {
-        &self.grid_auto_columns
+    fn grid_auto_columns(&self) -> Self::AutoTrackList<'_> {
+        self.grid_auto_columns.iter().copied()
     }
     #[inline(always)]
     fn grid_auto_flow(&self) -> GridAutoFlow {
@@ -849,25 +926,61 @@ impl GridContainerStyle for Style {
     fn justify_items(&self) -> Option<AlignItems> {
         self.justify_items
     }
+
+    #[inline(always)]
+    #[cfg(feature = "grid")]
+    fn grid_template_areas(&self) -> Option<Self::GridTemplateAreas<'_>> {
+        Some(self.grid_template_areas.iter().cloned())
+    }
+
+    #[inline(always)]
+    #[cfg(feature = "grid")]
+    fn grid_template_column_names(&self) -> Option<Self::TemplateLineNames<'_>> {
+        Some(self.grid_template_column_names.iter().map(|names| names.iter()))
+    }
+
+    #[inline(always)]
+    #[cfg(feature = "grid")]
+    fn grid_template_row_names(&self) -> Option<Self::TemplateLineNames<'_>> {
+        Some(self.grid_template_row_names.iter().map(|names| names.iter()))
+    }
 }
 
 #[cfg(feature = "grid")]
 impl<T: GridContainerStyle> GridContainerStyle for &'_ T {
+    type Repetition<'a>
+        = T::Repetition<'a>
+    where
+        Self: 'a;
+
     type TemplateTrackList<'a>
         = T::TemplateTrackList<'a>
     where
         Self: 'a;
+
     type AutoTrackList<'a>
         = T::AutoTrackList<'a>
     where
         Self: 'a;
 
+    /// The type returned by grid_template_row_names and grid_template_column_names
+    #[cfg(feature = "grid")]
+    type TemplateLineNames<'a>
+        = T::TemplateLineNames<'a>
+    where
+        Self: 'a;
+    #[cfg(feature = "grid")]
+    type GridTemplateAreas<'a>
+        = T::GridTemplateAreas<'a>
+    where
+        Self: 'a;
+
     #[inline(always)]
-    fn grid_template_rows(&self) -> Self::TemplateTrackList<'_> {
+    fn grid_template_rows(&self) -> Option<Self::TemplateTrackList<'_>> {
         (*self).grid_template_rows()
     }
     #[inline(always)]
-    fn grid_template_columns(&self) -> Self::TemplateTrackList<'_> {
+    fn grid_template_columns(&self) -> Option<Self::TemplateTrackList<'_>> {
         (*self).grid_template_columns()
     }
     #[inline(always)]
@@ -878,6 +991,21 @@ impl<T: GridContainerStyle> GridContainerStyle for &'_ T {
     fn grid_auto_columns(&self) -> Self::AutoTrackList<'_> {
         (*self).grid_auto_columns()
     }
+    #[cfg(feature = "grid")]
+    #[inline(always)]
+    fn grid_template_areas(&self) -> Option<Self::GridTemplateAreas<'_>> {
+        (*self).grid_template_areas()
+    }
+    #[cfg(feature = "grid")]
+    #[inline(always)]
+    fn grid_template_column_names(&self) -> Option<Self::TemplateLineNames<'_>> {
+        (*self).grid_template_column_names()
+    }
+    #[cfg(feature = "grid")]
+    #[inline(always)]
+    fn grid_template_row_names(&self) -> Option<Self::TemplateLineNames<'_>> {
+        (*self).grid_template_row_names()
+    }
     #[inline(always)]
     fn grid_auto_flow(&self) -> GridAutoFlow {
         (*self).grid_auto_flow()
@@ -905,14 +1033,16 @@ impl<T: GridContainerStyle> GridContainerStyle for &'_ T {
 }
 
 #[cfg(feature = "grid")]
-impl GridItemStyle for Style {
+impl<S: CheapCloneStr> GridItemStyle for Style<S> {
     #[inline(always)]
-    fn grid_row(&self) -> Line<GridPlacement> {
-        self.grid_row
+    fn grid_row(&self) -> Line<GridPlacement<S>> {
+        // TODO: Investigate eliminating clone
+        self.grid_row.clone()
     }
     #[inline(always)]
-    fn grid_column(&self) -> Line<GridPlacement> {
-        self.grid_column
+    fn grid_column(&self) -> Line<GridPlacement<S>> {
+        // TODO: Investigate eliminating clone
+        self.grid_column.clone()
     }
     #[inline(always)]
     fn align_self(&self) -> Option<AlignSelf> {
@@ -927,11 +1057,11 @@ impl GridItemStyle for Style {
 #[cfg(feature = "grid")]
 impl<T: GridItemStyle> GridItemStyle for &'_ T {
     #[inline(always)]
-    fn grid_row(&self) -> Line<GridPlacement> {
+    fn grid_row(&self) -> Line<GridPlacement<Self::CustomIdent>> {
         (*self).grid_row()
     }
     #[inline(always)]
-    fn grid_column(&self) -> Line<GridPlacement> {
+    fn grid_column(&self) -> Line<GridPlacement<Self::CustomIdent>> {
         (*self).grid_column()
     }
     #[inline(always)]
@@ -946,7 +1076,10 @@ impl<T: GridItemStyle> GridItemStyle for &'_ T {
 
 #[cfg(test)]
 mod tests {
+    use std::sync::Arc;
+
     use super::Style;
+    use crate::sys::DefaultCheapStr;
     use crate::{geometry::*, style_helpers::TaffyAuto as _};
 
     #[test]
@@ -954,7 +1087,8 @@ mod tests {
         #[cfg(feature = "grid")]
         use super::GridPlacement;
 
-        let old_defaults = Style {
+        let old_defaults: Style<DefaultCheapStr> = Style {
+            dummy: core::marker::PhantomData,
             display: Default::default(),
             item_is_table: false,
             item_is_replaced: false,
@@ -1000,6 +1134,12 @@ mod tests {
             #[cfg(feature = "grid")]
             grid_template_columns: Default::default(),
             #[cfg(feature = "grid")]
+            grid_template_row_names: Default::default(),
+            #[cfg(feature = "grid")]
+            grid_template_column_names: Default::default(),
+            #[cfg(feature = "grid")]
+            grid_template_areas: Default::default(),
+            #[cfg(feature = "grid")]
             grid_auto_rows: Default::default(),
             #[cfg(feature = "grid")]
             grid_auto_columns: Default::default(),
@@ -1011,7 +1151,7 @@ mod tests {
             grid_column: Line { start: GridPlacement::Auto, end: GridPlacement::Auto },
         };
 
-        assert_eq!(Style::DEFAULT, Style::default());
+        assert_eq!(Style::DEFAULT, Style::<DefaultCheapStr>::default());
         assert_eq!(Style::DEFAULT, old_defaults);
     }
 
@@ -1020,6 +1160,7 @@ mod tests {
     #[test]
     fn style_sizes() {
         use super::*;
+        type S = crate::sys::DefaultCheapStr;
 
         fn assert_type_size<T>(expected_size: usize) {
             let name = ::core::any::type_name::<T>();
@@ -1070,16 +1211,20 @@ mod tests {
         assert_type_size::<GridAutoFlow>(1);
         assert_type_size::<MinTrackSizingFunction>(8);
         assert_type_size::<MaxTrackSizingFunction>(8);
-        assert_type_size::<NonRepeatedTrackSizingFunction>(16);
-        assert_type_size::<TrackSizingFunction>(32);
-        assert_type_size::<Vec<NonRepeatedTrackSizingFunction>>(24);
+        assert_type_size::<TrackSizingFunction>(16);
         assert_type_size::<Vec<TrackSizingFunction>>(24);
+        assert_type_size::<Vec<GridTemplateComponent<S>>>(24);
 
-        // CSS Grid Item
-        assert_type_size::<GridPlacement>(4);
-        assert_type_size::<Line<GridPlacement>>(8);
+        // String-type dependent (String)
+        assert_type_size::<GridTemplateComponent<String>>(56);
+        assert_type_size::<GridPlacement<String>>(32);
+        assert_type_size::<Line<GridPlacement<String>>>(64);
+        assert_type_size::<Style<String>>(536);
 
-        // Overall
-        assert_type_size::<Style>(352);
+        // String-type dependent (Arc<str>)
+        assert_type_size::<GridTemplateComponent<Arc<str>>>(56);
+        assert_type_size::<GridPlacement<Arc<str>>>(24);
+        assert_type_size::<Line<GridPlacement<Arc<str>>>>(48);
+        assert_type_size::<Style<Arc<str>>>(504);
     }
 }
