@@ -15,6 +15,24 @@ pub(crate) use self::core::*;
 /// For when `std` is enabled
 #[cfg(feature = "std")]
 mod std {
+    // // Re-exporting a macro_rules macro doesn't work properly, so we wrap
+    // // it in a trivial new macro that just forwards it's input to the underlying
+    // // std/alloc macro
+    // macro_rules! format {
+    //     ($($tokens:tt)*) => {
+    //         ::std::format!($($tokens)*)
+    //     };
+    // }
+    // pub(crate) use format;
+
+    pub(crate) use std::format;
+
+    /// A string
+    pub(crate) type String = std::string::String;
+    /// The default type for representing strings in Taffy styles
+    pub(crate) type DefaultCheapStr = String;
+    /// A map
+    pub(crate) type Map<K, V> = std::collections::HashMap<K, V, std::collections::hash_map::RandomState>;
     /// An allocation-backend agnostic vector type
     pub(crate) type Vec<A> = std::vec::Vec<A>;
     /// A vector of child nodes
@@ -29,6 +47,12 @@ mod std {
         Vec::with_capacity(capacity)
     }
 
+    /// Creates a new vector with the capacity for the specified number of items before it must be resized
+    #[must_use]
+    pub(crate) fn single_value_vec<A>(value: A) -> Vec<A> {
+        vec![value]
+    }
+
     /// Rounds to the nearest whole number
     #[must_use]
     #[inline(always)]
@@ -76,6 +100,25 @@ mod alloc {
     extern crate alloc;
     use core::cmp::Ordering;
 
+    // // Re-exporting a macro_rules macro doesn't work properly, so we wrap
+    // // it in a trivial new macro that just forwards it's input to the underlying
+    // // std/alloc macro
+    // macro_rules! format {
+    //     ($($tokens:tt)*) => {
+    //         ::alloc::fmt::format!($($tokens)*)
+    //     };
+    // }
+    // pub(crate) use format;
+
+    pub(crate) use alloc::format;
+
+    /// A string
+    pub(crate) type String = alloc::string::String;
+    /// The default type for representing strings in Taffy styles
+    pub(crate) type DefaultCheapStr = String;
+    /// A map
+    // TODO: consider using hashbrown
+    pub(crate) type Map<K, V> = alloc::collections::BTreeMap<K, V>;
     /// An allocation-backend agnostic vector type
     pub(crate) type Vec<A> = alloc::vec::Vec<A>;
     /// A vector of child nodes
@@ -90,6 +133,14 @@ mod alloc {
         Vec::with_capacity(capacity)
     }
 
+    /// Creates a new vector with the capacity for the specified number of items before it must be resized
+    #[must_use]
+    pub(crate) fn single_value_vec<A>(value: A) -> Vec<A> {
+        let mut vec = Vec::with_capacity(1);
+        vec.push(value);
+        vec
+    }
+
     /// Rounds to the nearest whole number
     pub(crate) use super::polyfill::round;
 
@@ -128,6 +179,11 @@ mod core {
     /// The maximum number of children of any given node
     pub const MAX_GRID_TRACKS: usize = 16;
 
+    /// A string
+    pub(crate) type String = &'static str;
+    /// The default type for representing strings in Taffy styles
+    pub(crate) type DefaultCheapStr = &'static str;
+
     /// An allocation-backend agnostic vector type
     pub(crate) type Vec<A> = arrayvec::ArrayVec<A, MAX_NODE_COUNT>;
     /// A vector of child nodes, whose length cannot exceed [`MAX_CHILD_COUNT`]
@@ -144,6 +200,14 @@ mod core {
         arrayvec::ArrayVec::new()
     }
 
+    /// Creates a new vector with the capacity for the specified number of items before it must be resized
+    #[must_use]
+    pub(crate) fn single_value_vec<A, const CAP: usize>(value: A) -> arrayvec::ArrayVec<A, CAP> {
+        let mut vec = new_vec_with_capacity(1);
+        vec.push(value);
+        vec
+    }
+
     /// Rounds to the nearest whole number
     pub(crate) use super::polyfill::round;
 
