@@ -13,10 +13,7 @@ use self::{
     },
     message::{IncompleteMessage, IncompleteMessageType},
 };
-use crate::{
-    error::{CapacityError, Error, ProtocolError, Result},
-    protocol::frame::Utf8Bytes,
-};
+use crate::error::{Error, ProtocolError, Result};
 use log::*;
 use std::{
     io::{self, Read, Write},
@@ -33,28 +30,11 @@ pub enum Role {
 }
 
 /// The configuration for WebSocket connection.
-///
-/// # Example
-/// ```
-/// # use tungstenite::protocol::WebSocketConfig;;
-/// let conf = WebSocketConfig::default()
-///     .read_buffer_size(256 * 1024)
-///     .write_buffer_size(256 * 1024);
-/// ```
 #[derive(Debug, Clone, Copy)]
-#[non_exhaustive]
 pub struct WebSocketConfig {
-    /// Read buffer capacity. This buffer is eagerly allocated and used for receiving
-    /// messages.
-    ///
-    /// For high read load scenarios a larger buffer, e.g. 128 KiB, improves performance.
-    ///
-    /// For scenarios where you expect a lot of connections and don't need high read load
-    /// performance a smaller buffer, e.g. 4 KiB, would be appropriate to lower total
-    /// memory usage.
-    ///
-    /// The default value is 128 KiB.
-    pub read_buffer_size: usize,
+    /// Does nothing, instead use `max_write_buffer_size`.
+    #[deprecated]
+    pub max_send_queue: Option<usize>,
     /// The target minimum size of the write buffer to reach before writing the data
     /// to the underlying stream.
     /// The default value is 128 KiB.
@@ -94,8 +74,9 @@ pub struct WebSocketConfig {
 
 impl Default for WebSocketConfig {
     fn default() -> Self {
-        Self {
-            read_buffer_size: 128 * 1024,
+        #[allow(deprecated)]
+        WebSocketConfig {
+            max_send_queue: None,
             write_buffer_size: 128 * 1024,
             max_write_buffer_size: usize::MAX,
             max_message_size: Some(64 << 20),
@@ -106,42 +87,6 @@ impl Default for WebSocketConfig {
 }
 
 impl WebSocketConfig {
-    /// Set [`Self::read_buffer_size`].
-    pub fn read_buffer_size(mut self, read_buffer_size: usize) -> Self {
-        self.read_buffer_size = read_buffer_size;
-        self
-    }
-
-    /// Set [`Self::write_buffer_size`].
-    pub fn write_buffer_size(mut self, write_buffer_size: usize) -> Self {
-        self.write_buffer_size = write_buffer_size;
-        self
-    }
-
-    /// Set [`Self::max_write_buffer_size`].
-    pub fn max_write_buffer_size(mut self, max_write_buffer_size: usize) -> Self {
-        self.max_write_buffer_size = max_write_buffer_size;
-        self
-    }
-
-    /// Set [`Self::max_message_size`].
-    pub fn max_message_size(mut self, max_message_size: Option<usize>) -> Self {
-        self.max_message_size = max_message_size;
-        self
-    }
-
-    /// Set [`Self::max_frame_size`].
-    pub fn max_frame_size(mut self, max_frame_size: Option<usize>) -> Self {
-        self.max_frame_size = max_frame_size;
-        self
-    }
-
-    /// Set [`Self::accept_unmasked_frames`].
-    pub fn accept_unmasked_frames(mut self, accept_unmasked_frames: bool) -> Self {
-        self.accept_unmasked_frames = accept_unmasked_frames;
-        self
-    }
-
     /// Panic if values are invalid.
     pub(crate) fn assert_valid(&self) {
         assert!(
@@ -213,7 +158,7 @@ impl<Stream> WebSocket<Stream> {
     /// # Panics
     /// Panics if config is invalid e.g. `max_write_buffer_size <= write_buffer_size`.
     pub fn set_config(&mut self, set_func: impl FnOnce(&mut WebSocketConfig)) {
-        self.context.set_config(set_func);
+        self.context.set_config(set_func)
     }
 
     /// Read the configuration.
@@ -378,8 +323,7 @@ impl WebSocketContext {
     /// # Panics
     /// Panics if config is invalid e.g. `max_write_buffer_size <= write_buffer_size`.
     pub fn new(role: Role, config: Option<WebSocketConfig>) -> Self {
-        let conf = config.unwrap_or_default();
-        Self::_new(role, FrameCodec::new(conf.read_buffer_size), conf)
+        Self::_new(role, FrameCodec::new(), config.unwrap_or_default())
     }
 
     /// Create a WebSocket context that manages an post-handshake stream.
@@ -387,8 +331,7 @@ impl WebSocketContext {
     /// # Panics
     /// Panics if config is invalid e.g. `max_write_buffer_size <= write_buffer_size`.
     pub fn from_partially_read(part: Vec<u8>, role: Role, config: Option<WebSocketConfig>) -> Self {
-        let conf = config.unwrap_or_default();
-        Self::_new(role, FrameCodec::from_partially_read(part, conf.read_buffer_size), conf)
+        Self::_new(role, FrameCodec::from_partially_read(part), config.unwrap_or_default())
     }
 
     fn _new(role: Role, mut frame: FrameCodec, config: WebSocketConfig) -> Self {
@@ -467,7 +410,7 @@ impl WebSocketContext {
             // If we get here, either write blocks or we have nothing to write.
             // Thus if read blocks, just let it return WouldBlock.
             if let Some(message) = self.read_message_frame(stream)? {
-                trace!("Received message {message}");
+                trace!("Received message {}", message);
                 return Ok(message);
             }
         }
@@ -496,7 +439,7 @@ impl WebSocketContext {
         }
 
         let frame = match message {
-            Message::Text(data) => Frame::message(data, OpCode::Data(OpData::Text), true),
+            Message::Text(data) => Frame::message(data.into(), OpCode::Data(OpData::Text), true),
             Message::Binary(data) => Frame::message(data, OpCode::Data(OpData::Binary), true),
             Message::Ping(data) => Frame::ping(data),
             Message::Pong(data) => {
@@ -598,15 +541,13 @@ impl WebSocketContext {
     }
 
     /// Try to decode one message frame. May return None.
-    fn read_message_frame(&mut self, stream: &mut impl Read) -> Result<Option<Message>> {
-        if let Some(frame) = self
+    fn read_message_frame<Stream>(&mut self, stream: &mut Stream) -> Result<Option<Message>>
+    where
+        Stream: Read + Write,
+    {
+        if let Some(mut frame) = self
             .frame
-            .read_frame(
-                stream,
-                self.config.max_frame_size,
-                matches!(self.role, Role::Server),
-                self.config.accept_unmasked_frames,
-            )
+            .read_frame(stream, self.config.max_frame_size)
             .check_connection_reset(self.state)?
         {
             if !self.state.can_read() {
@@ -624,9 +565,26 @@ impl WebSocketContext {
                 }
             }
 
-            if self.role == Role::Client && frame.is_masked() {
-                // A client MUST close a connection if it detects a masked frame. (RFC 6455)
-                return Err(Error::Protocol(ProtocolError::MaskedFrameFromServer));
+            match self.role {
+                Role::Server => {
+                    if frame.is_masked() {
+                        // A server MUST remove masking for data frames received from a client
+                        // as described in Section 5.3. (RFC 6455)
+                        frame.apply_mask()
+                    } else if !self.config.accept_unmasked_frames {
+                        // The server MUST close the connection upon receiving a
+                        // frame that is not masked. (RFC 6455)
+                        // The only exception here is if the user explicitly accepts given
+                        // stream by setting WebSocketConfig.accept_unmasked_frames to true
+                        return Err(Error::Protocol(ProtocolError::UnmaskedFrameFromClient));
+                    }
+                }
+                Role::Client => {
+                    if frame.is_masked() {
+                        // A client MUST close a connection if it detects a masked frame. (RFC 6455)
+                        return Err(Error::Protocol(ProtocolError::MaskedFrameFromServer));
+                    }
+                }
             }
 
             match frame.header().opcode {
@@ -645,14 +603,14 @@ impl WebSocketContext {
                             Err(Error::Protocol(ProtocolError::UnknownControlFrameType(i)))
                         }
                         OpCtl::Ping => {
-                            let data = frame.into_payload();
+                            let data = frame.into_data();
                             // No ping processing after we sent a close frame.
                             if self.state.is_active() {
                                 self.set_additional(Frame::pong(data.clone()));
                             }
                             Ok(Some(Message::Ping(data)))
                         }
-                        OpCtl::Pong => Ok(Some(Message::Pong(frame.into_payload()))),
+                        OpCtl::Pong => Ok(Some(Message::Pong(frame.into_data()))),
                     }
                 }
 
@@ -661,7 +619,7 @@ impl WebSocketContext {
                     match data {
                         OpData::Continue => {
                             if let Some(ref mut msg) = self.incomplete {
-                                msg.extend(frame.into_payload(), self.config.max_message_size)?;
+                                msg.extend(frame.into_data(), self.config.max_message_size)?;
                             } else {
                                 return Err(Error::Protocol(
                                     ProtocolError::UnexpectedContinueFrame,
@@ -676,25 +634,23 @@ impl WebSocketContext {
                         c if self.incomplete.is_some() => {
                             Err(Error::Protocol(ProtocolError::ExpectedFragment(c)))
                         }
-                        OpData::Text if fin => {
-                            check_max_size(frame.payload().len(), self.config.max_message_size)?;
-                            Ok(Some(Message::Text(frame.into_text()?)))
-                        }
-                        OpData::Binary if fin => {
-                            check_max_size(frame.payload().len(), self.config.max_message_size)?;
-                            Ok(Some(Message::Binary(frame.into_payload())))
-                        }
                         OpData::Text | OpData::Binary => {
-                            let message_type = match data {
-                                OpData::Text => IncompleteMessageType::Text,
-                                OpData::Binary => IncompleteMessageType::Binary,
-                                _ => panic!("Bug: message is not text nor binary"),
+                            let msg = {
+                                let message_type = match data {
+                                    OpData::Text => IncompleteMessageType::Text,
+                                    OpData::Binary => IncompleteMessageType::Binary,
+                                    _ => panic!("Bug: message is not text nor binary"),
+                                };
+                                let mut m = IncompleteMessage::new(message_type);
+                                m.extend(frame.into_data(), self.config.max_message_size)?;
+                                m
                             };
-                            let mut incomplete = IncompleteMessage::new(message_type);
-                            incomplete
-                                .extend(frame.into_payload(), self.config.max_message_size)?;
-                            self.incomplete = Some(incomplete);
-                            Ok(None)
+                            if fin {
+                                Ok(Some(msg.complete()?))
+                            } else {
+                                self.incomplete = Some(msg);
+                                Ok(None)
+                            }
                         }
                         OpData::Reserved(i) => {
                             Err(Error::Protocol(ProtocolError::UnknownDataFrameType(i)))
@@ -715,8 +671,8 @@ impl WebSocketContext {
 
     /// Received a close frame. Tells if we need to return a close frame to the user.
     #[allow(clippy::option_option)]
-    fn do_close(&mut self, close: Option<CloseFrame>) -> Option<Option<CloseFrame>> {
-        debug!("Received close frame: {close:?}");
+    fn do_close<'t>(&mut self, close: Option<CloseFrame<'t>>) -> Option<Option<CloseFrame<'t>>> {
+        debug!("Received close frame: {:?}", close);
         match self.state {
             WebSocketState::Active => {
                 self.state = WebSocketState::ClosedByPeer;
@@ -725,7 +681,7 @@ impl WebSocketContext {
                     if !frame.code.is_allowed() {
                         CloseFrame {
                             code: CloseCode::Protocol,
-                            reason: Utf8Bytes::from_static("Protocol violation"),
+                            reason: "Protocol violation".into(),
                         }
                     } else {
                         frame
@@ -733,7 +689,7 @@ impl WebSocketContext {
                 });
 
                 let reply = Frame::close(close.clone());
-                debug!("Replying to close with {reply:?}");
+                debug!("Replying to close with {:?}", reply);
                 self.set_additional(reply);
 
                 Some(close)
@@ -765,7 +721,7 @@ impl WebSocketContext {
             }
         }
 
-        trace!("Sending frame: {frame:?}");
+        trace!("Sending frame: {:?}", frame);
         self.frame.buffer_frame(stream, frame).check_connection_reset(self.state)
     }
 
@@ -781,15 +737,6 @@ impl WebSocketContext {
     }
 }
 
-fn check_max_size(size: usize, max_size: Option<usize>) -> crate::Result<()> {
-    if let Some(max_size) = max_size {
-        if size > max_size {
-            return Err(Error::Capacity(CapacityError::MessageTooLong { size, max_size }));
-        }
-    }
-    Ok(())
-}
-
 /// The current connection state.
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 enum WebSocketState {
@@ -879,10 +826,10 @@ mod tests {
             0x03,
         ]);
         let mut socket = WebSocket::from_raw_socket(WriteMoc(incoming), Role::Client, None);
-        assert_eq!(socket.read().unwrap(), Message::Ping(vec![1, 2].into()));
-        assert_eq!(socket.read().unwrap(), Message::Pong(vec![3].into()));
+        assert_eq!(socket.read().unwrap(), Message::Ping(vec![1, 2]));
+        assert_eq!(socket.read().unwrap(), Message::Pong(vec![3]));
         assert_eq!(socket.read().unwrap(), Message::Text("Hello, World!".into()));
-        assert_eq!(socket.read().unwrap(), Message::Binary(vec![0x01, 0x02, 0x03].into()));
+        assert_eq!(socket.read().unwrap(), Message::Binary(vec![0x01, 0x02, 0x03]));
     }
 
     #[test]
