@@ -434,7 +434,12 @@ impl Default for DEV_QUERY_RESULT_ACTION_DATA_0 {
 }
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct HDEVQUERY(pub isize);
+pub struct HDEVQUERY(pub *mut core::ffi::c_void);
+impl HDEVQUERY {
+    pub fn is_invalid(&self) -> bool {
+        self.0.is_null()
+    }
+}
 impl Default for HDEVQUERY {
     fn default() -> Self {
         unsafe { core::mem::zeroed() }
